@@ -11,6 +11,7 @@
 #include <unistd.h>
 
 #if defined(HAVE_CLAMAV)
+
 # include <clamav.h>
 #else
 #warning No antivirus library defined, this module will not perform any virus scanning.
@@ -77,13 +78,15 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 	const gchar *path,
 	const gboolean recursive,
 	const gboolean executables_only,
-	const gboolean ignore_links
+	const gboolean ignore_links,
+	gint *stop_count
 );
 static gulong AVScanOPCalculateTotalFiles(
 	GList *paths_list,
 	const gboolean recursive,
 	const gboolean executables_only,
-	const gboolean ignore_links
+	const gboolean ignore_links,
+	gint *stop_count
 );
 
 gchar *AVScanGetAVEngineNameVersionString(void);
@@ -97,16 +100,16 @@ gint AVScanOP(
 );
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? (gint)strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? (gint)strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -364,7 +367,7 @@ static gchar *AVScanCopyShortenString(const gchar *s, const gint m)
 {
 	gint len;
 
-	if(s == NULL)  
+	if(s == NULL)
 	    return(NULL);
 
 	len = STRLEN(s);
@@ -396,7 +399,7 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	ClamAVData *d = (ClamAVData *)g_malloc0(
 	    sizeof(ClamAVData)
 	);
-	if(d == NULL)  
+	if(d == NULL)
 	{
 	    g_print("Memory allocation error\n");
 	    return(NULL);
@@ -448,7 +451,7 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	    }
 
 	    memset(&d->database_stat, 0x00, sizeof(struct cl_stat));
-	    cl_statinidir(d->database_dir, &d->database_stat);     
+	    cl_statinidir(d->database_dir, &d->database_stat);
 
 	    status = cl_loaddbdir(
 		d->database_path, &d->database, &d->database_entries
@@ -457,7 +460,7 @@ static ClamAVData *AVScanOPClamAVInit(const gchar *db_path)
 	else
 	{
 	    d->database_dir = g_dirname(d->database_path);
-	    if(d->database_dir == NULL)  
+	    if(d->database_dir == NULL)
 	    {
 		g_print("Memory allocation error\n");
 		g_free(d->database_path);
@@ -522,7 +525,7 @@ static void AVScanOPClamAVShutdown(ClamAVData *d)
 }
 
 /*
- *	Use Clam AntiVirus to scan the specified file.
+ *	Use the Clam AntiVirus to scan the specified file.
  */
 static gint AVScanOPClamAVScanFile(
 	ClamAVData *d, const gchar *path,
@@ -535,6 +538,10 @@ static gint AVScanOPClamAVScanFile(
 	gulong size_mb = 0l;
 	struct cl_limits limits;
 
+	/* Interrupted? */
+	if(*stop_count > 0)
+	    return(4);
+
 	PRINT_SCANNING_FILE(d->files_scanned, d->total_files, path);
 
 	/* Open the file for reading */
@@ -542,12 +549,12 @@ static gint AVScanOPClamAVScanFile(
 	if(fd < 0)
 	{
 	    const gint error_code = errno;
-	    gchar *error_msg = STRDUP(strerror((int)error_code));
+	    gchar *error_msg = STRDUP(g_strerror(error_code));
 	    if(error_msg != NULL)
 	    {
 		gchar *s;
 
-		*error_msg = toupper(*error_msg);
+		*error_msg = (gchar)toupper((int)*error_msg);
 		s = g_strdup_printf(
 		    "%s",
 		    error_msg
@@ -558,7 +565,10 @@ static gint AVScanOPClamAVScanFile(
 	    }
 	    else
 	    {
-		PRINT_ERR(path, "Unable to open the file for reading");
+		PRINT_ERR(
+		    path,
+		    "Unable to open the file for reading"
+		);
 	    }
 	    return(1);
 	}
@@ -601,7 +611,7 @@ static gint AVScanOPClamAVScanFile(
 	    options
 	);
 
-	/* Close file */
+	/* Close the file */
 	close(fd);
 
 	/* Count this file as scanned and how much scanned */
@@ -628,7 +638,11 @@ static gint AVScanOPClamAVScanFile(
 	/* Clean? */
 	else if(status == 0)
 	{
-	    return(0);
+	    /* Interrupted? */
+	    if(*stop_count > 0)
+		return(4);
+	    else
+		return(0);
 	}
 	/* Other error */
 	else
@@ -636,7 +650,7 @@ static gint AVScanOPClamAVScanFile(
 	    gchar *s, *error_msg = STRDUP(cl_strerror((int)status));
 	    if(error_msg != NULL)
 	    {
-		*error_msg = toupper(*error_msg);
+		*error_msg = (gchar)toupper((int)*error_msg);
 		s = g_strdup_printf(
 		    "%s",
 		    error_msg
@@ -666,16 +680,22 @@ static gint AVScanOPClamAVScanDir(
 	gchar *child;
 	struct dirent *dent;
 	struct stat stat_buf;
-	DIR *dp = opendir(path);
+	DIR *dp;
+
+	/* Interrupted? */
+	if(*stop_count > 0)
+	    return(4);
+
+	/* Open the directory */
+	dp = opendir(path);
 	if(dp == NULL)
 	{
 	    const gint error_code = errno;
-	    gchar *error_msg = STRDUP(strerror((int)error_code));
+	    gchar *error_msg = STRDUP(g_strerror(error_code));
 	    if(error_msg != NULL)
 	    {
 		gchar *s;
-
-		*error_msg = toupper(*error_msg);
+		*error_msg = (gchar)toupper((int)*error_msg);
 		s = g_strdup_printf(
 		    "%s",
 		    error_msg
@@ -691,6 +711,13 @@ static gint AVScanOPClamAVScanDir(
 	    return(1);
 	}
 
+	/* Interrupted? */
+	if(*stop_count > 0)
+	{
+	    closedir(dp);
+	    return(4);
+	}
+
 	/* Scan directory */
 	PRINT_SCANNING_DIR(path);
 	status = 0;
@@ -700,7 +727,7 @@ static gint AVScanOPClamAVScanDir(
 	    if(dent == NULL)
 		break;
 
-	    s = dent->d_name;
+	    s = (const gchar *)dent->d_name;
 
 	    /* Check for virus database change */
 	    if(cl_statchkdir(&d->database_stat) == 1)
@@ -741,28 +768,27 @@ static gint AVScanOPClamAVScanDir(
 		cl_statinidir(d->database_dir, &d->database_stat);
 	    }
 
-	    /* Skip special directory notations */
-	    if(!g_strcasecmp(s, ".") ||
-	       !g_strcasecmp(s, "..")
+	    /* Skip the current and parent directory notations */
+	    if(!strcmp((const char *)s, ".") ||
+	       !strcmp((const char *)s, "..")
 	    )
 		continue;
 
 	    /* Get the full path to the child object */
-	    child = g_strdup_printf(
-		"%s%c%s",
-		path, DIR_DELIMINATOR, s
-	    );
+	    child = STRDUP(PrefixPaths(
+		(const char *)path, (const char *)s
+	    ));
 	    if(child == NULL)
 		continue;
 
 	    /* Get this object's local stats */
-	    if(lstat(child, &stat_buf))
+	    if(lstat((const char *)child, &stat_buf))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *error_msg = STRDUP(g_strerror((int)error_code));
+		gchar *error_msg = STRDUP(g_strerror(error_code));
 		if(error_msg != NULL)
 		{
-		    *error_msg = toupper(*error_msg);
+		    *error_msg = (gchar)toupper((int)*error_msg);
 		    PRINT_ERR(child, error_msg);
 		    g_free(error_msg);
 		}
@@ -792,15 +818,14 @@ static gint AVScanOPClamAVScanDir(
 		    continue;
 		}
 
-		/* Get the destination value and see if it is a
-		 * reference to the current directory or parent
-		 * directory
+		/* Get the destination value and check if it is a
+		 * reference to the current or parent directory
 		 */
 		dest = (gchar *)GetAllocLinkDest(child);
 		if(dest != NULL)
 		{
-		    if(!g_strcasecmp(dest, ".") ||
-		       !g_strcasecmp(dest, "..")
+		    if(!strcmp((const char *)dest, ".") ||
+		       !strcmp((const char *)dest, "..")
 		    )
 		    {
 			/* Skip this to prevent infinite recursion */
@@ -813,13 +838,13 @@ static gint AVScanOPClamAVScanDir(
 	    }
 
 	    /* Get this object's destination stats */
-	    if(stat(child, &stat_buf))
+	    if(stat((const char *)child, &stat_buf))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *error_msg = STRDUP(g_strerror((int)error_code));
+		gchar *error_msg = STRDUP(g_strerror(error_code));
 		if(error_msg != NULL)
 		{
-		    *error_msg = toupper(*error_msg);
+		    *error_msg = (gchar)toupper((int)*error_msg);
 		    PRINT_ERR(child, error_msg);
 		    g_free(error_msg);
 		}
@@ -861,9 +886,10 @@ static gint AVScanOPClamAVScanDir(
 	    g_free(child);
 	}
 
+	/* Close the directory */
 	closedir(dp);
 
-	/* Iterrupted? */
+	/* Need to set status to interrupted? */
 	if((status == 0) && (*stop_count > 0))
 	    status = 4;
 
@@ -886,43 +912,43 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 	const gchar *path,
 	const gboolean recursive,
 	const gboolean executables_only,
-	const gboolean ignore_links
+	const gboolean ignore_links,
+	gint *stop_count
 )
 {
 	mode_t m;
 	gulong subtotal = 0l;
 	gchar *child;
-	const char *s;
+	const gchar *s;
 	struct dirent *dent;
 	struct stat stat_buf;
 	DIR *dp = opendir(path);
 	if(dp == NULL)
 	    return(subtotal);
 
-	while(TRUE)
+	while(*stop_count == 0)
 	{
 	    dent = readdir(dp);
 	    if(dent == NULL)
 		break;
 
-	    s = dent->d_name;
+	    s = (const gchar *)dent->d_name;
 
-	    /* Skip special directory notations */
-	    if(!g_strcasecmp(s, ".") ||
-	       !g_strcasecmp(s, "..")
+	    /* Skip the current and parent directory notations */
+	    if(!strcmp((const char *)s, ".") ||
+	       !strcmp((const char *)s, "..")
 	    )
 		continue;
 
 	    /* Get the full path to the child object */
-	    child = g_strdup_printf(
-		"%s%c%s",
-		path, DIR_DELIMINATOR, s   
-	    );
+	    child = STRDUP(PrefixPaths(
+		(const char *)path, (const char *)s   
+	    ));
 	    if(child == NULL)
 		continue;
 
 	    /* Get the object's local stats */
-	    if(lstat(child, &stat_buf))
+	    if(lstat((const char *)child, &stat_buf))
 	    {
 		/* Unable to get the local stats, do not count this */
 		g_free(child);
@@ -943,20 +969,19 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 		/* Ignore links? */
 		if(ignore_links)
 		{
-		    /* This is a link and it is ignored, do not count this */
+		    /* Do not count this */
 		    g_free(child);
 		    continue;
 		}
 
-		/* Get the destination value and see if it is a
-		 * reference to the current directory or parent
-		 * directory
+		/* Get the destination value and check if it is a
+		 * reference to the current or parent directory
 		 */
-		dest = (gchar *)GetAllocLinkDest(child);
+		dest = (gchar *)GetAllocLinkDest((const char *)child);
 		if(dest != NULL)
 		{
-		    if(!g_strcasecmp(dest, ".") ||
-		       !g_strcasecmp(dest, "..")
+		    if(!strcmp((const char *)dest, ".") ||
+		       !strcmp((const char *)dest, "..")
 		    )
 		    {
 			/* Skip this to prevent infinite recursion */
@@ -969,7 +994,7 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 	    }
 
 	    /* Get this object's destination stats */
-	    if(stat(child, &stat_buf))  
+	    if(stat((const char *)child, &stat_buf))
 	    {
 		/* Unable to get the destination stats, do not count this */
 		g_free(child);
@@ -978,7 +1003,9 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 
 	    m = stat_buf.st_mode;
 
-	    /* Directory and recursive? */
+	    /* Is the destination a directory and should we recurse
+	     * into it?
+	     */
 #ifdef S_ISDIR
 	    if(S_ISDIR(m) && recursive)
 #else
@@ -987,7 +1014,8 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 	    {
 		subtotal += AVScanOPCalculateTotalFilesInDirIterate(
 		    child,
-		    recursive, executables_only, ignore_links
+		    recursive, executables_only, ignore_links,
+		    stop_count
 		);
 	    }
 	    /* Executable only or all? */
@@ -1017,7 +1045,8 @@ static gulong AVScanOPCalculateTotalFiles(
 	GList *paths_list,
 	const gboolean recursive,
 	const gboolean executables_only,
-	const gboolean ignore_links
+	const gboolean ignore_links,
+	gint *stop_count
 )
 {
 	gulong total = 0l;
@@ -1032,12 +1061,16 @@ static gulong AVScanOPCalculateTotalFiles(
 	/* Iterate through each object in the paths list */
 	for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
 	{
+	    /* Interrupted */
+	    if(*stop_count > 0)
+		break;
+
 	    path = (const gchar *)glist->data;
 	    if(STRISEMPTY(path))
 		continue;
 
 	    /* Get this object's local stats */
-	    if(lstat(path, &stat_buf))
+	    if(lstat((const char *)path, &stat_buf))
 	    {
 		/* Unable to get local stats, do not count this */
 		continue;
@@ -1055,12 +1088,12 @@ static gulong AVScanOPCalculateTotalFiles(
 		/* Links ignored? */
 		if(ignore_links)
 		{
-		    /* This is a link and links are ignored, do not count this */
+		    /* Do not count this */
 		    continue;
 		}
 
 		/* Get this link's destination stats */
-		if(stat(path, &stat_buf))
+		if(stat((const char *)path, &stat_buf))
 		{
 		    /* Unable to get destination stats, do not count this */
 		    continue;
@@ -1068,17 +1101,23 @@ static gulong AVScanOPCalculateTotalFiles(
 
 		m = stat_buf.st_mode;
 
+		/* Is this link's destination a directory? */
 #ifdef S_ISDIR
 		if(S_ISDIR(m))
 #else
 		if(FALSE)
 #endif
+		{
 		    total += AVScanOPCalculateTotalFilesInDirIterate(
 			path,
-			recursive, executables_only, ignore_links
+			recursive, executables_only, ignore_links,
+			stop_count
 		    );
+		}
 		else
+		{
 		    total++;
+		}
 	    }
 	    /* Directory? */
 #ifdef S_ISDIR
@@ -1089,7 +1128,8 @@ static gulong AVScanOPCalculateTotalFiles(
 	    {
 		total += AVScanOPCalculateTotalFilesInDirIterate(
 		    path,
-		    recursive, executables_only, ignore_links
+		    recursive, executables_only, ignore_links,
+		    stop_count
 		);
 	    }
 	    /* File? */
@@ -1114,68 +1154,11 @@ static gulong AVScanOPCalculateTotalFiles(
 gchar *AVScanGetAVEngineNameVersionString(void)
 {
 #if defined(HAVE_CLAMAV)
-	const gchar	*config_prog = PREFIX "/bin/clamav-config",
-			*tmp_dir;
-	gint p, fd;
-	gchar *cmd, *stdout_path, *version_str, *name_version_str;
-	FILE *fp;
-
-	/* Get tempory files directory */
-	tmp_dir = g_getenv(ENV_VAR_NAME_TMPDIR);
-	if(STRISEMPTY(tmp_dir))
-#if defined(P_tmpdir)
-	    tmp_dir = P_tmpdir;
-#elif defined(__MSW__)
-	    tmp_dir = "C:\\TEMP";
-#else
-	    tmp_dir = "/tmp";
-#endif
-	stdout_path = STRDUP(PrefixPaths(
-	    tmp_dir,
-	    PROG_NAME "XXXXXX"
-	));
-	fd = (gint)mkstemp((char *)stdout_path);
-	if(fd > -1)
-	    close((int)fd);
-
-	cmd = g_strdup_printf(
-	    "\"%s\" --version",
-	    config_prog
-	);
-	p = ExecBO(cmd, stdout_path);
-	g_free(cmd);
-
-	if(p <= 0)
-	{
-	    unlink(stdout_path);
-	    g_free(stdout_path);
-	    return(NULL);
-	}
-
-	fp = FOpen(stdout_path, "rb");
-	if(fp != NULL)
-	{
-	    version_str = FGetString(fp);
-	    FClose(fp);
-	}
-	else
-	{
-	    unlink(stdout_path);
-	    g_free(stdout_path);
-	    return(NULL);
-	}
-
-	name_version_str = g_strdup_printf(
+	return(g_strdup_printf(
 	    "%s Version %s",
 	    "Clam AntiVirus",
-	    version_str
-	);
-	g_free(version_str);
-
-	unlink(stdout_path);
-	g_free(stdout_path);
-
-	return(name_version_str);
+	    cl_retver()
+	));
 #else
 	return(NULL);
 #endif
@@ -1186,6 +1169,11 @@ gchar *AVScanGetAVEngineNameVersionString(void)
  *	AntiVirus Scan Operation
  *
  *	This is the front end to performing scans.
+ *
+ *	The stop_count is a pointer to a gint which should be
+ *	initialized to 0 prior to this call. This value will be checked
+ *	throughout this module's functions and if (*stop_count > 0) then
+ *	it will skip all scanning related calls.
  */
 gint AVScanOP(
 	const gchar *db_path,
@@ -1228,12 +1216,18 @@ gint AVScanOP(
 
 	    /* Calculate the total files to be scanned */
 	    d->total_files = AVScanOPCalculateTotalFiles(
-		paths_list, recursive, executables_only, ignore_links
+		paths_list,
+		recursive, executables_only, ignore_links,
+		stop_count
 	    );
 
 	    /* Iterate through each object in the paths list */
 	    for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
 	    {
+		/* Interrupted */
+		if(*stop_count > 0)
+		    break;
+
 		path = (const gchar *)glist->data;
 		if(STRISEMPTY(path))
 		    continue;
@@ -1243,10 +1237,10 @@ gint AVScanOP(
 		{
 		    /* Unable to get this object's local stats */
 		    const gint error_code = (gint)errno;
-		    gchar *error_msg = STRDUP(g_strerror((int)error_code));
+		    gchar *error_msg = STRDUP(g_strerror(error_code));
 		    if(error_msg != NULL)
 		    {
-			*error_msg = toupper(*error_msg);
+			*error_msg = (gchar)toupper((int)*error_msg);
 			PRINT_ERR(path, error_msg);
 			g_free(error_msg);
 		    }
@@ -1277,10 +1271,10 @@ gint AVScanOP(
 		    {
 			/* Unable to get this link's destination stats */
 			const gint error_code = (gint)errno;
-			gchar *error_msg = STRDUP(g_strerror((int)error_code));
+			gchar *error_msg = STRDUP(g_strerror(error_code));
 			if(error_msg != NULL)
 			{
-			    *error_msg = toupper(*error_msg);
+			    *error_msg = (gchar)toupper((int)*error_msg);
 			    PRINT_ERR(path, error_msg);
 			    g_free(error_msg);
 			}
@@ -1289,21 +1283,26 @@ gint AVScanOP(
 
 		    m = stat_buf.st_mode;
 
+		    /* Is this link's destination a directory? */
 #ifdef S_ISDIR
 		    if(S_ISDIR(m))
 #else
 		    if(FALSE)
 #endif
+		    {
 			status2 = AVScanOPClamAVScanDir(
 			    d, path,
 			    recursive, executables_only, ignore_links,
 			    stop_count
 			);
+		    }
 		    else
+		    {
 			status2 = AVScanOPClamAVScanFile(
 			    d, path,
 			    stop_count
 			);
+		    }
 		}
 		/* Directory? */
 #ifdef S_ISDIR
@@ -1337,13 +1336,20 @@ gint AVScanOP(
 		if((status2 != 0) && (status == 0))
 		    status = status2;
 	    }
-	    PRINT_REPORT(
-		time_start,
-		(gulong)time(NULL),
-		d->infected_files,
-		d->size_scanned_mb,
-		d->files_scanned
-	    );
+
+	    /* Not interrupted? */
+	    if(*stop_count == 0)
+	    {
+		/* Print the final report */
+		PRINT_REPORT(
+		    time_start,
+		    (gulong)time(NULL),
+		    d->infected_files,
+		    d->size_scanned_mb,
+		    d->files_scanned
+		);
+	    }
+
 	    AVScanOPClamAVShutdown(d);
 	}
 	else
@@ -1351,9 +1357,16 @@ gint AVScanOP(
 	    status = 3;
 	}
 #else
-	g_print("No AntiVirus support enabled at compile time\n");
+	PRINT_ERR(
+	    NULL,
+	    "No AntiVirus support enabled at compile time"
+	);
 	status = 1;
 #endif
 
+	/* Need to set status to interrupted? */
+	if((status == 0) && (*stop_count > 0))
+	    status = 4;
+
 	return(status);
 }
