@@ -9,8 +9,9 @@ extern gchar *AVScanGetAVEngineNameVersionString(void);
 extern gint AVScanOP(
 	const gchar *db_path,
 	GList *path_list,
-	gboolean recursive,
-	gboolean executables_only,
+	const gboolean recursive,
+	const gboolean executables_only,
+	const gboolean ignore_links,
 	gint *stop_count
 );
 
