@@ -21,11 +21,11 @@
 #define PROG_NAME		"AVScan"
 #define PROG_NAME_FULL		"AntiVirus Scanner"
 
-#define PROG_VERSION		"0.8.3"
+#define PROG_VERSION		"1.1.2"
 
-#define PROG_VERSION_MAJOR	0
-#define PROG_VERSION_MINOR	8
-#define PROG_VERSION_RELEASE	3
+#define PROG_VERSION_MAJOR	1
+#define PROG_VERSION_MINOR	1
+#define PROG_VERSION_RELEASE	2
 
 /*
  *	Program URL:
@@ -107,13 +107,13 @@ to the fullest extent of the law.\n"
  *	Usage message:
  */
 #define PROG_USAGE_MESG  "\
-Usage: avscan [paths...] [options]\n\
-       avscan --scan <paths...> [options]\n\
+Usage: avscan <path(s)...> [options]\n\
+       avscan --scan <path(s)...> [options]\n\
 \n\
-    [path(s)...] specifies the file(s) or directory(ies) to scan at\n\
-    startup.\n\
+    The <path(s)...> specifies the full paths to the file(s) and/or\n\
+    directory(ies) to scan at startup.\n\
 \n\
-    [options] can be any of the following:\n\
+    The [options] can be any of the following:\n\
 \n\
         --recursive             Recurse into subdirectories.\n\
         -r                      Same as --recursive.\n\
@@ -126,18 +126,16 @@ Usage: avscan [paths...] [options]\n\
         -db <path>              Same as --database.\n\
         --help                  Prints (this) help screen and exits.\n\
         --version               Prints version information and exits.\n\
-\n\
-    <path(s)...> specifies the file(s) and/or directory(ies) to scan\n\
-    in text mode (used with --scan only).\n\
 \n\
     Return values:\n\
 \n\
         0       Success.\n\
         1       Virus detected or general error.\n\
-        2       Bad value.\n\
-        3       Systems error.\n\
+        2       Invalid value.\n\
+        3       Systems error or memory allocation error.\n\
         4       User aborted.\n\
-        5       An identical operation is already in progress.\n\
+        5       User responded with \"no\" to a query.\n\
+        6       An identical operation is already in progress.\n\
         8       Segmentation fault.\n\
 \n"
 
