@@ -7,6 +7,7 @@
 #endif
 #include "guiutils.h"
 #include "cfg.h"
+#include "win.h"
 
 
 typedef struct _core_struct     core_struct;
@@ -60,4 +61,7 @@ struct _core_struct {
 #define CORE(p)		((core_struct *)(p))
 
 
+extern win_struct *MainNewAVScan(core_struct *core, gint argc, gchar **argv); 
+
+
 #endif	/* CORE_H */
