@@ -7,7 +7,7 @@
 #include "../include/os.h"
 
 
-#if defined(WIN32)
+#if defined(_WIN32)
 	#include <io.h>	/* Needed by _findfirst(), _findnext(), etc */
 #else
 	#include <unistd.h>
@@ -17,7 +17,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#if !defined(USE_GETDENTS) && !defined(WIN32)
+#if !defined(USE_GETDENTS) && !defined(_WIN32)
 	#include <dirent.h>
 #endif
 
@@ -103,7 +103,7 @@ int FILEHASEXTENSION(const char *filename)
  */
 int ISPATHABSOLUTE(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	if( path == NULL )
 		return(0);
 
@@ -137,7 +137,7 @@ int ISPATHABSOLUTE(const char *path)
  */
 int NUMDIRCONTENTS(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	int i, total = 0;
 	const char *s;
 	char **strv = GetDirEntNames(path);
@@ -260,7 +260,7 @@ int ISPATHDIR(const char *path)
 
 int ISLPATHDIR(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	return(ISPATHDIR(path));
 #else
 	struct stat stat_buf;
@@ -278,7 +278,7 @@ int ISLPATHDIR(const char *path)
 
 int ISPATHEXECUTABLE(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	char *s;
 
 	if( STRISEMPTY(path) )
@@ -332,68 +332,106 @@ int rmkdir(const char *path, mode_t m)
 	char *s, *full_path;
 	struct stat stat_buf;
 
-	if( STRISEMPTY(path) )
-		return(-1);
+	if(STRISEMPTY(path))
+	{
+#ifdef EINVAL
+	    errno = EINVAL;
+#endif
+	    return(-1);
+	}
 
 	/* Need to get full absolute path */
-	if( ISPATHABSOLUTE(path) )
+	if(ISPATHABSOLUTE(path))
 	{
-		full_path = STRDUP(path);
-	} else
-	{
-		char cwd[PATH_MAX];
-
-		/* Not absolute path, so get current directory and prefix
-		 * it to the specified path
-		 */
-		if( getcwd(cwd, sizeof(cwd)) != NULL )
-			cwd[sizeof(cwd) - 1] = '\0';
-		else
-			return(-1);
-
-		full_path = STRDUP(PrefixPaths(cwd, path));
+	    full_path = STRDUP(path);
 	}
-	if( full_path == NULL )
-		return(-3);
+	else
+	{
+	    char cwd[PATH_MAX];
 
-	/* At this point, full_path is a copy of the completed path
-	 *
-	 * Next we will modify full_path by replacing each deliminator
-	 * with a null byte to check if the path compoent exists or not
-	 * so we know if we need to create it
+	    /* Not an absolute path
+	     *
+	     * Get the current working directory and prefix it to the
+	     * specified path
+	     */
+	    if(getcwd(cwd, sizeof(cwd)) != NULL)
+		cwd[sizeof(cwd) - 1] = '\0';
+	    else
+		return(-1);
+
+	    full_path = STRDUP(PrefixPaths(cwd, path));
+	}
+	if(full_path == NULL)
+	{
+#ifdef ENOMEM
+	    errno = ENOMEM;
+#endif
+	    return(-3);
+	}
+
+	/* Begin creating each directory compoent by iterating through
+	 * each path compoent in full_path
 	 */
 
-	/* Seek to start of full_path + 1 to skip the first deliminator */
-	s = full_path + 1;
-	while( s != NULL )
-	{
-		/* Seek to next deliminator and set it null temporarly to
-		 * check if the path compoent exists or not
-		 */
-		s = strchr(s, DIR_DELIMINATOR);
-		if( s != NULL )
-			*s = '\0';
-
-		if( stat(full_path, &stat_buf) )
-		{
-#ifdef WIN32
-			if( mkdir(full_path) )
-			{
-				free(full_path);
-				return(-1);
-			}
+	/* Seek s past the toplevel deliminator in full_path */
+#if defined(_WIN32)
+	s = strchr(full_path, DIR_DELIMINATOR);
+	if(s != NULL)
+	    s++;
+	else
+	    s = full_path;
 #else
-			if( mkdir(full_path, m) )
-			{
-				free(full_path);
-				return(-1);
-			}
+	s = full_path + 1;
+#endif
+	while(s != NULL)
+	{
+	    /* Seek s to the next path compoent and temporarly set
+	     * its deliminator to null
+	     */
+	    s = strchr(s, DIR_DELIMINATOR);
+	    if(s != NULL)
+		*s = '\0';
+
+	    /* Does this directory need to be created? */
+	    if(stat(full_path, &stat_buf) != 0)
+	    {
+		const int error_code = errno;
+		if(error_code == ENOENT)
+		{
+#if defined(_WIN32)
+		    if(mkdir(full_path))
+		    {
+			const int error_code = errno;
+			free(full_path);
+			errno = error_code;
+			return(-1);
+		    }
+#else
+		    if(mkdir(full_path, m))
+		    {
+			const int error_code = errno;
+			free(full_path);
+			errno = error_code;
+			return(-1);
+		    }
 #endif
 		}
+		else
+		{
+		    free(full_path);
+		    errno = error_code;
+		    return(-1);
+		}
+	    }
 
-		/* Set the pointer back to the deliminator and increment */
-		if( s != NULL )
-			*s++ = DIR_DELIMINATOR;
+	    /* Set back the directory deliminator and seek s to the
+	     * next directory compoent
+	     */
+	    if(s != NULL)
+	    {
+		*s = DIR_DELIMINATOR;
+		s++;
+	    }
 	}
 
 	free(full_path);
@@ -409,7 +447,7 @@ int rmkdir(const char *path, mode_t m)
  */
 char *PathSubHome(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	static char rtn_path[PATH_MAX];
 
 	if( STRISEMPTY(path) )
@@ -578,7 +616,7 @@ char **GetDirEntNames2(const char *parent, int *total)
 
 char **GetDirEntNames2(const char *parent, int *total)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	long ffh;		/* Find file handle */
 	struct _finddata_t d;	/* Find data return structure */
 	char prev_cwd[PATH_MAX];
@@ -708,7 +746,7 @@ char **GetDirEntNames2(const char *parent, int *total)
 		*total = i;
 
 	return(rtn_names);
-#endif	/* NOT WIN32 */
+#endif	/* NOT _WIN32 */
 }
 #endif /* USE_GETDENTS */
 
@@ -1040,7 +1078,7 @@ char *PrefixPaths(const char *parent, const char *child)
  */
 char *GetAllocLinkDest(const char *link)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	return(NULL);
 #else
 	int bytes_read;
@@ -1079,7 +1117,7 @@ char *GetAllocLinkDest(const char *link)
 	dest[bytes_read] = '\0';
 
 	return(dest);
-#endif	/* NOT WIN32 */
+#endif	/* NOT _WIN32 */
 }
 
 
@@ -1107,7 +1145,7 @@ char *CompletePath(char *path, int *status)
 		 * to complete the path
 		 */
 		int i, strc, matches = 0;
-		char *name, **match = NULL;
+		char *name, **matches_list = NULL;
 		char **strv = GetDirEntNames2(GetParentDir(path), &strc);
 
 		/* Get match list containing all object names that have
@@ -1117,38 +1155,40 @@ char *CompletePath(char *path, int *status)
 		for( i = 0; i < strc; i++ )
 		{
 			name = strv[i];
-			if( name == NULL )
+			if(name == NULL)
 				continue;
 
-			if( !strcmp(name, ".") ||
-				!strcmp(name, "..")
-			  )
+			if(!strcmp(name, ".") ||
+			   !strcmp(name, "..")
+			)
 			{
 				free(name);
 				continue;
 			}
 
-			if( strpfx(name, cur_child) )
+			if(strpfx(name, cur_child))
 			{
 				/* Append this name to the matches list */
 				int n = matches;
 				matches = n + 1;
-				match = (char **)realloc(
-										match, matches * sizeof(char *)
-										);
-				match[n] = STRDUP(name);
+				matches_list = (char **)realloc(
+					matches_list,
+					matches * sizeof(char *)
+				);
+				matches_list[n] = STRDUP(name);
 			}
 
 			free(name);
 		}
 		free(strv);
 
-		/* Got one match? */
+		/* Got exactly one match? */
 		if( matches == 1 )
 		{
-			char *new_path = STRDUP(
-								   PrefixPaths(GetParentDir(path), match[0])
-								   );
+			char *new_path = STRDUP(PrefixPaths(
+				GetParentDir(path),
+				matches_list[0]
+			));
 			free(path);
 			path = new_path;
 			if( ISPATHDIR(path) )
@@ -1167,7 +1207,7 @@ char *CompletePath(char *path, int *status)
 			const char *ss, *st;
 			for( i = 0; i < matches; i++ )
 			{
-				ss = match[i];
+				ss = matches_list[i];
 				if( ss == NULL )
 					continue;
 
@@ -1176,7 +1216,7 @@ char *CompletePath(char *path, int *status)
 					if( n == i )
 						continue;
 
-					st = match[n];
+					st = matches_list[n];
 					if( st == NULL )
 						continue;
 
@@ -1204,18 +1244,19 @@ char *CompletePath(char *path, int *status)
 			{
 				if( status != NULL )
 					*status = COMPLETE_PATH_AMBIGUOUS;
-			} else
+			}
+			else
 			{
-				char    *new_path,
-				*new_child = STRDUP(match[0]);
+				char *new_path,
+				*new_child = STRDUP(matches_list[0]);
 				new_child[len] = '\0';
-				new_path = STRDUP(
-								 PrefixPaths(GetParentDir(path), new_child)
-								 );
+				new_path = STRDUP(PrefixPaths(
+				    GetParentDir(path), new_child
+				));
 				free(path);
 				path = new_path;
 				free(new_child);
-				if( status != NULL )
+				if(status != NULL)
 					*status = COMPLETE_PATH_PARTIAL;
 			}
 		}
@@ -1226,12 +1267,11 @@ char *CompletePath(char *path, int *status)
 				*status = COMPLETE_PATH_NONE;
 		}
 
-		/* Delete matched names list */
+		/* Delete the matched names list */
 		for( i = 0; i < matches; i++ )
-			free(match[i]);
-		free(match);
-	} else
-	{
+			free(matches_list[i]);
+		free(matches_list);
+	} else {
 		/* Child name is empty so get listing of object names and
 		 * only set match if there is exactly one object
 		 */
@@ -1245,15 +1285,15 @@ char *CompletePath(char *path, int *status)
 			if( name == NULL )
 				continue;
 
-			if( !strcmp(name, ".") ||  
-				!strcmp(name, "..")
-			  )
+			if(!strcmp(name, ".") ||  
+			   !strcmp(name, "..")
+			)
 			{
 				free(name);
 				continue;
 			}
 
-			if( match == NULL )
+			if(match == NULL)
 				match = STRDUP(name);
 			matches++;
 
@@ -1261,25 +1301,26 @@ char *CompletePath(char *path, int *status)
 		}
 		free(strv);
 
-		/* Got one match? */
+		/* Got exactly one match? */
 		if( matches == 1 )
 		{
-			char *new_path = STRDUP(
-								   PrefixPaths(path, match)
-								   );
+			char *new_path = STRDUP(PrefixPaths(
+				path, match
+			));
 			free(path);
 			path = new_path;
 			if( ISPATHDIR(path) )
 				path = strcatalloc(path, "/");
 			if( status != NULL )
 				*status = COMPLETE_PATH_SUCCESS;
-		} else
+		}
+		else
 		{
 			if( status != NULL )
 				*status = COMPLETE_PATH_AMBIGUOUS;
 		}
 
-		/* Delete matched name */
+		/* Delete the matched name */
 		free(match);
 	}
 
@@ -1322,7 +1363,7 @@ int FileCountLines(const char *filename)
  */
 int DirHasSubDirs(const char *path)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
 	return(0);
 #else
 	int status = 0;
@@ -1382,7 +1423,7 @@ int DirHasSubDirs(const char *path)
 	closedir(dir);
 
 	return(status);
-#endif	/* NOT WIN32 */
+#endif	/* NOT _WIN32 */
 }
 
 
@@ -1409,7 +1450,7 @@ char *GetParentDir(const char *path)
 		return(parent);
 
 	/* Strip tailing deliminators */
-#ifdef WIN32
+#ifdef _WIN32
 	while( s >= (parent + 3) )
 #else
 	while( s >= (parent + 1) )
@@ -1423,7 +1464,7 @@ char *GetParentDir(const char *path)
 	}
 
 	/* Get parent */
-#ifdef WIN32
+#ifdef _WIN32
 	if( s >= (parent + 3) )
 		*s = '\0';
 	else
@@ -1595,7 +1636,7 @@ int CopyObject(
 	    return(-2);
 
 	/* Check if target exists */
-#ifdef WIN32
+#ifdef _WIN32
 	if(!stat(target, &stat_buf))
 #else
 	if(!lstat(target, &stat_buf))
