@@ -15,8 +15,6 @@
 #endif
 #include <unistd.h>
 
-#include "../include/disk.h"
-
 #include "guiutils.h"
 #include "cdialog.h"
 #include "pdialog.h"
@@ -25,7 +23,7 @@
 
 #include "avscanop.h"
 #include "cfg.h"
-#include "cfgfio.h"
+#include "cfg_fio.h"
 #include "obj.h"
 #include "objfio.h"
 #include "win.h"
@@ -42,28 +40,27 @@
 #include "images/icon_virus_pattern_20x20.xpm"
 
 
-static gint		*lstop_count_ptr;
-static core_struct	*lcore_ptr;	/* Pointer to core */
-
-
 static void MainSignalCB(int s);
 static gint MainTOCB(gpointer data);
 
 static cfg_item_struct *MainCfgListNew(void);
 
-static win_struct *MainWinNew(core_struct *core);
+win_struct *MainNewAVScan(core_struct *core, gint argc, gchar **argv);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+static gint	*avscan_scan_stop_count;
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
+
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -71,37 +68,38 @@ static win_struct *MainWinNew(core_struct *core);
  */
 static void MainSignalCB(int s)
 {
-	gint *stop_count = lstop_count_ptr;
-	core_struct *core = lcore_ptr;
-
 	switch(s)
 	{
 	  case SIGINT:
-	  case SIGTERM:
-	    if(core != NULL)
+	    /* Is this process running as a scan process? */
+	    if(avscan_scan_stop_count != NULL)
 	    {
-		core->close_all_windows = TRUE;
-	    }
-	    else if(stop_count != NULL)
-	    {
-#if 0
+		gint *stop_count = avscan_scan_stop_count;
+
+		/* Increment the scan's stop count so that the scan
+		 * process will stop nicely
+		 *
+		 * When its stop count is increment it will wait for
+		 * the current (if any) ClamAV call to finish and
+		 * continue to execute without calling any additional
+		 * ClamAV calls except AVScanOPClamAVShutdown()
+		 * in order to shut down ClamAV nicely (so that there
+		 * are no ClamAV tempory files) and finish execution
+		 */
 		*stop_count = (*stop_count) + 1;
-		/* If 5 or more stop counts then do explicit exit */
-		if(*stop_count >= 5)
-		    exit(4);
-#else
-/* Just exit for now, stop count checking is not immediate enough */
-		exit(4);
-#endif
 	    }
 	    else
 	    {
-		exit(4);
+		exit(4);	/* User aborted */
 	    }
 	    break;
 
+	  case SIGTERM:
+	    exit(4);	/* User aborted */
+	    break;
+
 	  case SIGSEGV:
-	    exit(8);
+	    exit(8);	/* Segmentation fault */
 	    break;
 
 	  case SIGSTOP:
@@ -193,7 +191,7 @@ static cfg_item_struct *MainCfgListNew(void)
 	CFGItemListSetValueL(
 	    cfg_list, CFG_PARM_DB_LAST_UPDATE,
 	    0l, FALSE
-	);                           
+	);
  
 	CFGItemListSetValueI(
 	    cfg_list, CFG_PARM_WIN_X,
@@ -202,7 +200,7 @@ static cfg_item_struct *MainCfgListNew(void)
 	CFGItemListSetValueI(
 	    cfg_list, CFG_PARM_WIN_Y,
 	    0, FALSE
-	);   
+	);
 	CFGItemListSetValueI(
 	    cfg_list, CFG_PARM_WIN_SHOW_TOOL_BAR,
 	    TRUE, FALSE
@@ -220,7 +218,7 @@ static cfg_item_struct *MainCfgListNew(void)
 /*
  *	Creates a new Win.
  */
-static win_struct *MainWinNew(core_struct *core)
+win_struct *MainNewAVScan(core_struct *core, gint argc, gchar **argv)
 {
 	gchar *path;
 	GList *glist;
@@ -228,25 +226,30 @@ static win_struct *MainWinNew(core_struct *core)
 	if(win == NULL)
 	    return(NULL);
 
+	GUIWindowApplyArgs(GTK_WINDOW(win->toplevel), argc, argv);
+
 	/* Clear the scan settings */
 	WinScanSettingsUpdate(win, NULL);
 
 	/* Reset status progress, message, and time */
 	WinStatusProgress(win, 0.0f, FALSE);
 	WinStatusMessage(win, "Ready", FALSE);
-        WinStatusTime(win, 0l, FALSE);
+	WinStatusTime(win, 0l, FALSE);
 
 	/* Map the window */
 	WinMap(win);
 
 	WinSetBusy(win, TRUE);
 
-        /* Open the scan list from file */
-        path = STRDUP(PrefixPaths(
-            core->prog_data_dir, AVSCAN_SCAN_LIST_FILE
-        ));
-        WinScanListOpenFromFile(win, path);
-        g_free(path);
+	/* Open the scan list from file */
+	path = g_strconcat(
+	    core->prog_data_dir,
+	    G_DIR_SEPARATOR_S,
+	    AVSCAN_SCAN_LIST_FILE,
+	    NULL
+	);
+	WinScanListOpenFromFile(win, path);
+	g_free(path);
 
 	WinSetBusy(win, FALSE);
 
@@ -272,8 +275,8 @@ int main(int argc, char *argv[])
 {
 	gboolean initialized_gtk = FALSE;
 	gint i;
-	const gchar *s, *arg;
-	gchar *path, cwd[PATH_MAX];
+	const gchar *s, *arg, *cwd;
+	gchar *path;
 	GdkWindow *root;
 	cfg_item_struct *cfg_list;
 #ifdef HAVE_EDV2
@@ -282,15 +285,10 @@ int main(int argc, char *argv[])
 	win_struct *win;
 	core_struct *core;
 
+	/* Initialize the globals */
+	avscan_scan_stop_count = NULL;
 
-	/* Reset globals */
-	lstop_count_ptr = NULL;
-	lcore_ptr = NULL;
-
-	/* Set up time zone */
-	tzset();
-
-	/* Set up signal callbacks */
+	/* Set up the signal callbacks */
 	signal(SIGINT, MainSignalCB);
 	signal(SIGTERM, MainSignalCB);
 	signal(SIGSEGV, MainSignalCB);
@@ -298,18 +296,16 @@ int main(int argc, char *argv[])
 	signal(SIGCONT, MainSignalCB);
 	signal(SIGPIPE, MainSignalCB);
 
-	/* Get current working directory */
-	if(getcwd(cwd, sizeof(cwd)) != NULL)
-	    cwd[sizeof(cwd) - 1] = '\0';
-	else
-	    *cwd = '\0';
-
+	/* Get the current working directory */
+	cwd = g_get_current_dir();
+	if(cwd == NULL)
+	   cwd = "/";
 
 	/* Parse arguments */
 	for(i = 1; i < argc; i++)
 	{
 	    arg = argv[i];
-	    if(arg == NULL)
+	    if(STRISEMPTY(arg))
 		continue;
 
 	    /* Help */
@@ -325,9 +321,7 @@ int main(int argc, char *argv[])
 	    }
 	    /* Version */
 	    else if(!g_strcasecmp(arg, "--version") ||
-		    !g_strcasecmp(arg, "-version") ||
-		    !g_strcasecmp(arg, "--v") ||
-		    !g_strcasecmp(arg, "-v")
+		    !g_strcasecmp(arg, "-version")
 	    )
 	    {
 		gchar *av_engine_name_version =
@@ -346,19 +340,16 @@ AntiVirus Engine: %s\n",
 	    /* Scan */
 	    else if(!g_strcasecmp(arg, "--scan"))
 	    {
-		/* Do scan procedure and exit */
-		gint status;
+		/* Do scan procedure and return */
+		gint status, stop_count = 0;
 		gchar *db_path = NULL;
 		GList *path_list = NULL;
 		gboolean	recursive = FALSE,
 				executables_only = FALSE,
 				ignore_links = FALSE;
 
-		/* Allocate and set up the stop counter */
-		if(lstop_count_ptr == NULL)
-		    lstop_count_ptr = (gint *)g_malloc(sizeof(gint));
-		if(lstop_count_ptr != NULL)
-		    *lstop_count_ptr = 0;
+		/* Set the stop count pointer for the signal callback */
+		avscan_scan_stop_count = &stop_count;
 
 		/* Collect subsequent arguments */
 		for(i++; i < argc; i++)
@@ -391,6 +382,7 @@ AntiVirus Engine: %s\n",
 		    }
 		    /* Executables Only */
 		    else if(!g_strcasecmp(arg, "--executables-only") ||
+		            !g_strcasecmp(arg, "--executables_only") ||
 			    !g_strcasecmp(arg, "-x")
 		    )
 		    {
@@ -398,17 +390,19 @@ AntiVirus Engine: %s\n",
 		    }
 		    /* Ignore Links */
 		    else if(!g_strcasecmp(arg, "--ignore-links") ||
-                            !g_strcasecmp(arg, "-l")
-                    )
+		            !g_strcasecmp(arg, "--ignore_links") ||
+			    !g_strcasecmp(arg, "-l")
+		    )
 		    {
 			ignore_links = TRUE;
 		    }
 		    /* All else assume path */
 		    else if((*arg != '-') && (*arg != '+'))
 		    {
-			path_list = g_list_append(
-			    path_list, STRDUP(arg)
-			);
+			gchar *path = g_path_is_absolute(arg) ?
+			    STRDUP(arg) :
+			    g_strconcat(cwd, G_DIR_SEPARATOR_S, arg, NULL);
+			path_list = g_list_append(path_list, path);
 		    }
 		}
 
@@ -419,17 +413,17 @@ AntiVirus Engine: %s\n",
 		    recursive,		/* Recursive */
 		    executables_only,	/* Executables Only */
 		    ignore_links,	/* Ignore Links */
-		    lstop_count_ptr
+		    &stop_count
 		);
 
+		/* Unset the stop count pointer */
+		avscan_scan_stop_count = NULL;
+
 		g_list_foreach(path_list, (GFunc)g_free, NULL);
 		g_list_free(path_list);
 
 		g_free(db_path);
 
-		g_free(lstop_count_ptr);
-		lstop_count_ptr = NULL;
-
 		return(status);
 	    }
 	}
@@ -446,7 +440,7 @@ AntiVirus Engine: %s\n",
 		return(1);
 	    }
 
-	    /* Initialize GDK RGB buffers */   
+	    /* Initialize GDK RGB buffers */
 	    gdk_rgb_init();
 
 	    initialized_gtk = TRUE;
@@ -462,7 +456,7 @@ AntiVirus Engine: %s\n",
 
 
 	/* Begin creating the core */
-	lcore_ptr = core = CORE(
+	core = CORE(
 	    g_malloc0(sizeof(core_struct))
 	);
 
@@ -502,7 +496,7 @@ AntiVirus Engine: %s\n",
 	{
 	    GdkColor	c_fg = { 0l, 0xffff, 0xffff, 0xffff },
 			c_bg = { 0l, 0x0000, 0x0000, 0x0000 };
-	    const gint width = 24, height = 24;   
+	    const gint width = 24, height = 24;
 	    const gchar color_mask_data[] = {
 0x03, 0x00, 0x00,
 0x05, 0x00, 0x00,
@@ -554,10 +548,10 @@ AntiVirus Engine: %s\n",
 0x00, 0xe0, 0xff,
 0x00, 0xe0, 0xff,
 0x00, 0xe0, 0xff
-	    };  
+	    };
 	    GdkBitmap *color_mask = gdk_bitmap_create_from_data(
 		root, color_mask_data, width, height
-	    );   
+	    );
 	    GdkBitmap *trans_mask = gdk_bitmap_create_from_data(
 		root, trans_mask_data, width, height
 	    );
@@ -587,13 +581,13 @@ AntiVirus Engine: %s\n",
 	EDVContextLoadConfigurationFile(ctx, NULL);
 #endif
 
-	/* Get full path to program */
-	arg = argv[0];
+	/* Get the full path to the program */
+	arg = (argc > 0) ? argv[0] : "*unknown*";
 	core->prog_name = STRDUP(arg);
 	core->prog_path = STRDUP(arg);
 
 #ifdef HAVE_EDV2
-	/* Get data directory */
+	/* Get the data directory */
 	s = EDVGetS(ctx, EDV_CFG_PARM_DIR_LOCAL);
 	if(s == NULL)
 	    s = g_getenv("HOME");
@@ -607,25 +601,28 @@ AntiVirus Engine: %s\n",
 	core->prog_data_dir = STRDUP(s);
 #endif
 
-	/* Load configuration from file */
-	path = STRDUP(PrefixPaths(
-	    core->prog_data_dir, AVSCAN_CFG_FILE
-	));
-	CFGLoadFromFile(path, cfg_list);
+	/* Open the configuration from file */
+	path = g_strconcat(
+	    core->prog_data_dir,
+	    G_DIR_SEPARATOR_S,
+	    AVSCAN_CFG_FILE,
+	    NULL
+	);
+	CFGFileOpen(path, cfg_list);
 	g_free(path);
 
 
-	/* Create first Win */
-	win = MainWinNew(core);
+	/* Create the AVScan window */
+	win = MainNewAVScan(core, argc, argv);
 	if(win != NULL)
 	{
-	    gboolean scan_on_startup = FALSE;
+	    GList *paths_list = NULL;
 
-	    /* Parse arguments and apply to the Win */
+	    /* Parse the arguments */
 	    for(i = 1; i < argc; i++)
 	    {
 		arg = argv[i];
-		if(arg == NULL)
+		if(STRISEMPTY(arg))
 		    continue;
 
 		/* Database Path */
@@ -654,10 +651,11 @@ AntiVirus Engine: %s\n",
 		    gtk_toggle_button_set_active(
 			GTK_TOGGLE_BUTTON(win->recursive_check),
 			TRUE 
-		    );      
+		    );
 		}
 		/* Executables Only */
 		else if(!g_strcasecmp(arg, "--executables-only") ||
+		        !g_strcasecmp(arg, "--executables_only") ||
 			!g_strcasecmp(arg, "-x")
 		)
 		{
@@ -667,7 +665,8 @@ AntiVirus Engine: %s\n",
 		    );
 		}
 		/* Ignore Links */
-		else if(!g_strcasecmp(arg, "--ignore_links") ||
+		else if(!g_strcasecmp(arg, "--ignore-links") ||
+		        !g_strcasecmp(arg, "--ignore_links") ||
 			!g_strcasecmp(arg, "-l")
 		)
 		{
@@ -676,25 +675,74 @@ AntiVirus Engine: %s\n",
 			TRUE
 		    );
 		}
+		/* Skip GUI arguments that have a an argument */
+		else if(!g_strcasecmp(arg, "--geometry") ||
+			!g_strcasecmp(arg, "-geometry") ||
+			!g_strcasecmp(arg, "--fn") ||
+			!g_strcasecmp(arg, "-fn") ||
+			!g_strcasecmp(arg, "--fg") ||
+			!g_strcasecmp(arg, "-fg") ||
+			!g_strcasecmp(arg, "--bg") ||
+			!g_strcasecmp(arg, "-bg") ||
+			!g_strcasecmp(arg, "--sfg") ||
+			!g_strcasecmp(arg, "-sfg") ||
+			!g_strcasecmp(arg, "--sbg") ||
+			!g_strcasecmp(arg, "-sbg") ||
+			!g_strcasecmp(arg, "--cfg") ||
+			!g_strcasecmp(arg, "-cfg") ||
+			!g_strcasecmp(arg, "--cbg") ||
+			!g_strcasecmp(arg, "-cbg") ||
+			!g_strcasecmp(arg, "--bg-pixmap") ||
+			!g_strcasecmp(arg, "-bg-pixmap") ||
+			!g_strcasecmp(arg, "--sbg-pixmap") ||
+			!g_strcasecmp(arg, "-sbg-pixmap") ||
+			!g_strcasecmp(arg, "--name") ||
+			!g_strcasecmp(arg, "-name") ||
+			!g_strcasecmp(arg, "--class") ||
+			!g_strcasecmp(arg, "-class") ||
+			!g_strcasecmp(arg, "--title") ||
+			!g_strcasecmp(arg, "-title")
+		)
+		{
+		    i++;
+		}
 		/* All else assume path */
 		else if((*arg != '-') && (*arg != '+'))
 		{
-		    gchar *path = STRDUP(ISPATHABSOLUTE(arg) ?
-			arg : PrefixPaths(cwd, arg)
-		    );
-		    gtk_entry_set_text(
-			GTK_ENTRY(win->location_entry),
-			path
-		    );
-		    g_free(path);
-
-		    scan_on_startup = TRUE;
+		    gchar *path = g_path_is_absolute(arg) ?
+			STRDUP(arg) :
+			g_strconcat(cwd, G_DIR_SEPARATOR_S, arg, NULL);
+		    paths_list = g_list_append(paths_list, path);
 		}
 	    }
 
 	    /* Need to start scan on startup? */
-	    if(scan_on_startup)
+	    if(paths_list != NULL)
+	    {
+		gchar *s, *location = STRDUP("");
+		GList *glist;
+		GtkEntry *entry = GTK_ENTRY(win->location_entry);
+
+		for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
+		{
+		    s = g_strconcat(
+			location,
+			(const gchar *)glist->data,
+			(g_list_next(glist) != NULL) ? "," : "",
+			NULL
+		    );
+		    g_free(location);
+		    location = s;
+		}
+		gtk_entry_set_text(entry, location);
+		gtk_entry_set_position(entry, -1);
+		g_free(location);
+
 		WinStartCB(win->start_btn, win);
+
+		g_list_foreach(paths_list, (GFunc)g_free, NULL);
+		g_list_free(paths_list);
+	    }
 	}
 
 	/* Set main timeout callback */
@@ -702,11 +750,14 @@ AntiVirus Engine: %s\n",
 	gtk_main();
 
 
-	/* Save configuration to file */
-	path = STRDUP(PrefixPaths(
-	    core->prog_data_dir, AVSCAN_CFG_FILE
-	));
-	CFGSaveToFile(path, cfg_list);
+	/* Save the configuration to file */
+	path = g_strconcat(
+	    core->prog_data_dir,
+	    G_DIR_SEPARATOR_S,
+	    AVSCAN_CFG_FILE,
+	    NULL
+	);
+	CFGFileSave(path, cfg_list);
 	g_free(path);
 
 
@@ -746,7 +797,6 @@ AntiVirus Engine: %s\n",
 	g_free(core->title);
 	g_free(core->geometry);
 
-	lcore_ptr = NULL;
 	g_free(core);
 	core = NULL;
 
