@@ -7,6 +7,18 @@
 typedef struct _obj_struct		obj_struct;
 
 
+/*
+ *	Scan options:
+ */
+typedef enum {
+
+	AVSCAN_OPT_RECURSIVE			= (1 << 0),
+	AVSCAN_OPT_EXECUTABLES_ONLY		= (1 << 1),
+	AVSCAN_OPT_IGNORE_LINKS			= (1 << 2)
+
+} avscan_options;
+
+
 /*
  *	Object:
  */
@@ -14,8 +26,7 @@ struct _obj_struct {
 
 	gchar		*name,
 			*path;
-	gboolean	recursive,
-			executables_only;
+	avscan_options	options;
 	gulong		last_runned;
 
 };
