@@ -6,6 +6,7 @@
 #include "../include/disk.h"
 
 #include "guiutils.h"
+#include "pulist.h"
 #include "pdialog.h"
 #include "config.h"
 
@@ -71,6 +72,7 @@ typedef enum {
 	PDIALOG_PROMPT_TYPE_ENTRY,
 	PDIALOG_PROMPT_TYPE_SPIN,
 	PDIALOG_PROMPT_TYPE_SCALE,
+	PDIALOG_PROMPT_TYPE_POPUP_LIST,
 	PDIALOG_PROMPT_TYPE_COMBO,
 	PDIALOG_PROMPT_TYPE_RADIO,
 	PDIALOG_PROMPT_TYPE_TOGGLE,
@@ -79,7 +81,9 @@ typedef enum {
 
 
 typedef struct _pdialog_struct		pdialog_struct;
+#define PDIALOG(p)			((pdialog_struct *)(p))
 typedef struct _pdialog_prompt_struct	pdialog_prompt_struct;
+#define PDIALOG_PROMPT(p)		((pdialog_prompt_struct *)(p))
 
 
 /*
@@ -94,20 +98,20 @@ struct _pdialog_prompt_struct {
 			*value,
 			*browse_btn;
 
+	pulistbox_struct *pulistbox;
 	GtkWidget	*radio_toplevel, **radio;
 	gchar		**radio_label;
 	gint		total_radios;
-  
-	gpointer	client_data;
 
 	gchar	*(*browse_cb)(
 		gpointer,	/* PDialog */
 		gpointer,	/* Data */
 		gint		/* Prompt Number */
 	);
+	gpointer	browse_data;
 
 };
-#define PDIALOG_PROMPT(p)	((pdialog_prompt_struct *)(p))
+
 
 /*
  *	Prompt Dialog:
@@ -139,7 +143,6 @@ struct _pdialog_struct {
 	gchar		**response_val;
 	gint		response_nvals;
 };
-#define PDIALOG(p)	((pdialog_struct *)(p))
 
 
 /* Callbacks */
@@ -166,14 +169,14 @@ static gboolean PDialogPromptDragMotionCB(
 static guint8 **PDialogGetMessageIconDataFromCode(pdialog_icon icon_code);
 static void PDialogSetIcon(pdialog_struct *d, guint8 **icon_data);
 static pdialog_prompt_struct *PDialogPromptNewNexus(
-	pdialog_prompt_type type,
+	const pdialog_prompt_type type,
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
-	gboolean hide_value,		/* For passwords */
+	const gboolean hide_value,	/* For passwords */
 	pdialog_struct *d,
-	gpointer client_data,
-	gchar *(*browse_cb)(gpointer, gpointer, gint)
+	gchar *(*browse_cb)(gpointer, gpointer, gint),
+	gpointer browse_data
 );
 static void PDialogPromptDelete(pdialog_prompt_struct *p);
 
@@ -186,13 +189,13 @@ void PDialogBreakQuery(void);
 GtkWidget *PDialogGetToplevel(void);
 
 static pdialog_prompt_struct *PDialogAddPromptNexus(
-	pdialog_prompt_type type,
+	const pdialog_prompt_type type,
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
-	gboolean hide_value,		/* For passwords */
-	gpointer client_data,
-	gchar *(*browse_cb)(gpointer, gpointer, gint)
+	const gboolean hide_value,	/* For passwords */
+	gchar *(*browse_cb)(gpointer, gpointer, gint),
+	gpointer browse_data
 );
 void PDialogAddPromptLabel(const gchar *label);
 void PDialogAddPrompt(
@@ -209,7 +212,7 @@ void PDialogAddPromptWithBrowse(
 	const guint8 **icon_data,	/* Can be NULL */
 	const gchar *label,		/* Can be NULL */
 	const gchar *value,		/* Can be NULL */
-	gpointer client_data,		/* Can be NULL */
+	gpointer browse_data,		/* Can be NULL */
 	gchar *(*browse_cb)(gpointer, gpointer, gint)	/* Can be NULL */
 );
 void PDialogAddPromptSpin(
@@ -226,18 +229,26 @@ void PDialogAddPromptScale(
 	gfloat step_increment, gfloat page_increment,
 	gboolean show_value, guint digits
 );
+void PDialogAddPromptPopupList(
+	const guint8 **icon_data,
+	const gchar *label,
+	GList *list,			/* List of values */
+	const gint start_num,		/* Initial value */
+	const gint nitems_visible	/* Number of items visible */
+);
 void PDialogAddPromptCombo(
 	const guint8 **icon_data,
 	const gchar *label,
-	const gchar *value,
-	GList *list,
-	gboolean editable, gboolean case_sensitive
+	const gchar *value,		/* Initial value */
+	GList *list,			/* List of values */
+	const gboolean editable,
+	const gboolean case_sensitive
 );
 void PDialogAddPromptRadio(
 	const guint8 **icon_data,
 	const gchar *label,
-	GList *list,		/* List of radio button names */
-	gint start_num		/* Initial radio button to select */
+	GList *list,			/* List of values */
+	const gint start_num		/* Initial value */
 );
 void PDialogAddPromptToggle(
 	const guint8 **icon_data,
@@ -342,7 +353,7 @@ static void PDialogBrowseButtonCB(GtkWidget *widget, gpointer data)
 	    {
 		s = (const gchar *)p->browse_cb(
 		    d,			/* Prompt dialog */
-		    p->client_data,	/* Client data */
+		    p->browse_data,	/* Data */
 		    i			/* Prompt number */
 		);
 		w = p->value;
@@ -414,6 +425,12 @@ static gint PDialogEntryCompletePathKeyCB(
 	switch(key->keyval)
 	{
 	  case GDK_Tab:
+	    /* Skip this if the shift or ctrl keys are held */
+	    if((key->state & GDK_CONTROL_MASK) ||
+	       (key->state & GDK_SHIFT_MASK)
+	    )
+		return(status);
+
 	    if(press)
 	    {
 		gchar *s = STRDUP(gtk_entry_get_text(entry));
@@ -461,7 +478,7 @@ static void PDialogButtonCB(GtkWidget *widget, gpointer data)
 	{
 	    gint i;
 
-	    /* Clear existing responses values */
+	    /* Clear the existing responses values */
 	    for(i = 0; i < d->response_nvals; i++)
 		g_free(d->response_val[i]);
 	    g_free(d->response_val);
@@ -542,6 +559,21 @@ static void PDialogButtonCB(GtkWidget *widget, gpointer data)
 			}
 			break;
 
+		      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+			if(p->pulistbox != NULL)
+			{
+			    pulistbox_struct *pulistbox = p->pulistbox;
+			    pulist_struct *pulist = PUListBoxGetPUList(pulistbox);
+			    gchar *v;
+			    PUListGetItemText(
+				pulist,
+				PUListGetSelectedLast(pulist),
+				&v
+			    );
+			    d->response_val[i] = STRDUP((v != NULL) ? v : "");
+			}
+			break;
+
 		      case PDIALOG_PROMPT_TYPE_COMBO:
 			if(GTK_IS_COMBO(w))
 			{
@@ -841,14 +873,14 @@ static void PDialogSetIcon(pdialog_struct *d, guint8 **icon_data)
  *	add it to the prompt dialog.
  */
 static pdialog_prompt_struct *PDialogPromptNewNexus(
-	pdialog_prompt_type type,
+	const pdialog_prompt_type type,
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
-	gboolean hide_value,		/* For passwords */
+	const gboolean hide_value,	/* For passwords */
 	pdialog_struct *d,
-	gpointer client_data,
-	gchar *(*browse_cb)(gpointer, gpointer, gint)
+	gchar *(*browse_cb)(gpointer, gpointer, gint),
+	gpointer browse_data
 )
 {
 	const gint	border_major = 5,
@@ -906,13 +938,15 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 	/* Primary input widget */
 	if(TRUE)
 	{
-	    GtkWidget *w2;
 	    const GtkTargetEntry dnd_tar_types[] = {
 {"text/plain",				0,	0},
 {"text/uri-list",			0,	1},
 {"STRING",				0,	2}
 	    };
-	    /* Create by prompt type */
+	    GtkWidget *w2;
+	    pulistbox_struct *pulistbox;
+
+	    /* Create the primary input widget by the prompt type */
 	    switch(type)
 	    {
 	      case PDIALOG_PROMPT_TYPE_LABEL:
@@ -968,6 +1002,14 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 		gtk_widget_show(w);
 		break;
 
+	      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+		p->pulistbox = pulistbox = PUListBoxNew(
+		    parent,
+		    -1, -1
+		);
+		PUListBoxMap(pulistbox);
+		break;
+
 	      case PDIALOG_PROMPT_TYPE_COMBO:
 		p->value = w = gtk_combo_new();
 		gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
@@ -1031,8 +1073,8 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 	if(browse_cb != NULL)
 	{
 	    /* Set browse callback function and client data */
-	    p->client_data = client_data;
 	    p->browse_cb = browse_cb;
+	    p->browse_data = browse_data;
 
 	    /* Create browse button */
 	    p->browse_btn = w = GUIButtonPixmap(
@@ -1099,11 +1141,13 @@ static void PDialogPromptDelete(pdialog_prompt_struct *p)
 	p->total_radios = 0;
 	GTK_WIDGET_DESTROY(p->radio_toplevel)
 
-	GTK_WIDGET_DESTROY(p->browse_btn)
-	GTK_WIDGET_DESTROY(p->value)
-	GTK_WIDGET_DESTROY(p->label)
-	GTK_WIDGET_DESTROY(p->icon_pm)
-	GTK_WIDGET_DESTROY(p->toplevel)
+	PUListBoxDelete(p->pulistbox);
+
+	GTK_WIDGET_DESTROY(p->browse_btn);
+	GTK_WIDGET_DESTROY(p->value);
+	GTK_WIDGET_DESTROY(p->label);
+	GTK_WIDGET_DESTROY(p->icon_pm);
+	GTK_WIDGET_DESTROY(p->toplevel);
 
 	g_free(p);
 }
@@ -1500,13 +1544,13 @@ GtkWidget *PDialogGetToplevel(void)
  *	Nexus for adding a prompt to the dialog.
  */
 static pdialog_prompt_struct *PDialogAddPromptNexus(
-	pdialog_prompt_type type,
+	const pdialog_prompt_type type,
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
-	gboolean hide_value,		/* For passwords */
-	gpointer client_data,
-	gchar *(*browse_cb)(gpointer, gpointer, gint)
+	const gboolean hide_value,		/* For passwords */
+	gchar *(*browse_cb)(gpointer, gpointer, gint),
+	gpointer browse_data
 )
 {
 	gint n;
@@ -1516,7 +1560,7 @@ static pdialog_prompt_struct *PDialogAddPromptNexus(
 	    type,
 	    icon_data, label, value, hide_value,
 	    d,
-	    client_data, browse_cb
+	    browse_cb, browse_data
 	);
 	if(p == NULL)
 	    return(NULL);
@@ -1611,13 +1655,13 @@ void PDialogAddPromptWithBrowse(
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
-	gpointer client_data,
+	gpointer browse_data,
 	gchar *(*browse_cb)(gpointer, gpointer, gint)
 )
 {
 	PDialogAddPromptNexus(
 	    PDIALOG_PROMPT_TYPE_ENTRY,
-	    icon_data, label, value, FALSE, client_data, browse_cb
+	    icon_data, label, value, FALSE, browse_cb, browse_data
 	);
 }
 
@@ -1685,17 +1729,87 @@ void PDialogAddPromptScale(
 	}
 }
 
+/*
+ *	Adds a popup list prompt to the dialog.
+ *
+ *	The list specifies the list of values, each value is a
+ *	gchar * to a string. The list will not be modified by this
+ *	function. If list is NULL then no list of values will be set.
+ *
+ *	The start_num specifies the initial value or -1 for none.
+ *
+ *	The nitems_visible specifies the number of items visible or
+ *	-1 for default.
+ */
+void PDialogAddPromptPopupList(
+	const guint8 **icon_data,
+	const gchar *label,
+	GList *list,			/* List of values */
+	const gint start_num,		/* Initial value */
+	const gint nitems_visible
+)
+{
+	pdialog_prompt_struct *p = PDialogAddPromptNexus(
+	    PDIALOG_PROMPT_TYPE_POPUP_LIST,
+	    icon_data, label, NULL, FALSE, NULL, NULL
+	);
+	if(p != NULL)
+	{
+	    pulistbox_struct *pulistbox = p->pulistbox;
+	    if(pulistbox != NULL)
+	    {
+		pulist_struct *pulist = PUListBoxGetPUList(pulistbox);
+		GList *glist = list;
+		while(glist != NULL)
+		{
+		    PUListAddItem(pulist, (gchar *)glist->data);
+		    glist = g_list_next(glist);
+		}
+		if(start_num > -1)
+		    PUListBoxSelect(pulistbox, start_num);
+
+		if(nitems_visible > -1)
+		{
+		    PUListBoxSetLinesVisible(
+			pulistbox, nitems_visible
+		    );
+		}
+		else
+		{
+		    PUListBoxSetLinesVisible(
+			pulistbox,
+			MIN(
+			    PUListGetTotalItems(pulist),
+			    10
+			)
+		    );
+		}
+	    }
+	}
+}
+
 /*
  *	Adds a combo prompt to the dialog.
  *
- *	The given glist is not modified by this function.
+ *	The value specifies the initial value. If value is NULL then
+ *	no initial value will be set.
+ *
+ *	The list specifies the list of values, each value is a
+ *	gchar * to a string. The list will not be modified by this
+ *	function. If list is NULL then no list of values will be set.
+ *
+ *	If editable is TRUE then the entry will be set as editable.
+ *
+ *	If case_sensitive is TRUE then only case sensitive matches
+ *	will be made with the list values and the entry value.
  */
 void PDialogAddPromptCombo(
 	const guint8 **icon_data,
 	const gchar *label,
 	const gchar *value,
 	GList *list,
-	gboolean editable, gboolean case_sensitive
+	const gboolean editable,
+	const gboolean case_sensitive
 )
 {
 	pdialog_prompt_struct *p = PDialogAddPromptNexus(
@@ -1719,13 +1833,17 @@ void PDialogAddPromptCombo(
 /*
  *	Adds a radio prompt to the dialog.
  *
- *	The specified glist is not modified by this function.
+ *	The list specifies the list of values, each value is a
+ *	gchar * to a string. The list will not be modified by this
+ *	function. If list is NULL then no list of values will be set.
+ *
+ *	The start_num specifies the initial value or -1 for none.
  */
 void PDialogAddPromptRadio(
 	const guint8 **icon_data,
 	const gchar *label,
-	GList *list,		/* List of radio button names */
-	gint start_num          /* Initial radio button to select */
+	GList *list,
+	const gint start_num
 )
 {
 	pdialog_prompt_struct *p = PDialogAddPromptNexus(
@@ -1762,7 +1880,8 @@ void PDialogAddPromptRadio(
 
 		/* Record value as a the radio button label string */
 		p->radio_label = (gchar **)g_realloc(
-		    p->radio_label, p->total_radios * sizeof(gchar *)
+		    p->radio_label,
+		    p->total_radios * sizeof(gchar *)
 		);
 		p->radio_label[i] = STRDUP((glist->data != NULL) ?
 		    (const gchar *)glist->data : ""
@@ -1909,6 +2028,30 @@ void PDialogSetPromptValue(
 		}
 		break;
 
+	      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+		if(p->pulistbox != NULL)
+		{
+		    gint i;
+		    gchar *v;
+		    pulistbox_struct *pulistbox = p->pulistbox;
+		    pulist_struct *pulist = PUListBoxGetPUList(pulistbox);
+		    const gint m = PUListGetTotalItems(pulist);
+
+		    for(i = 0; i < m; i++)
+		    {
+			PUListGetItemText(pulist, i, &v);
+			if(v == NULL)
+			    continue;
+
+			if(!g_strcasecmp(v, value))
+			{
+			    PUListBoxSelect(pulistbox, i);
+			    break;
+			}
+		    }
+		}
+		break;
+
 	      case PDIALOG_PROMPT_TYPE_COMBO:
 		if(GTK_IS_COMBO(w))
 		{
@@ -1968,7 +2111,6 @@ void PDialogSetPromptTip(gint prompt_num, const gchar *tip)
 	pdialog_prompt_struct *p;
 	pdialog_struct *d = &pdialog;
 
-
 	if(prompt_num < 0)
 	    prompt_num = d->total_prompts - 1;
 
@@ -1997,6 +2139,14 @@ void PDialogSetPromptTip(gint prompt_num, const gchar *tip)
 		GUISetWidgetTip(w, tip);
 		break;
 
+	      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+		if(p->pulistbox != NULL)
+		{
+		    pulistbox_struct *pulistbox = p->pulistbox;
+		    GUISetWidgetTip(pulistbox->da, tip);
+		}
+		break;
+
 	      case PDIALOG_PROMPT_TYPE_COMBO:
 		w = p->value;
 		if(w != NULL)
@@ -2088,6 +2238,21 @@ const gchar *PDialogGetPromptValue(gint prompt_num)
 		}
 		break;
 
+	      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+		if(p->pulistbox != NULL)
+		{
+		    gchar *v;
+		    pulistbox_struct *pulistbox = p->pulistbox;
+		    pulist_struct *pulist = PUListBoxGetPUList(pulistbox);
+		    PUListGetItemText(
+			pulist,
+			PUListGetSelectedLast(pulist),
+			&v
+		    );
+		    return(v);
+		}
+		break;
+
 	      case PDIALOG_PROMPT_TYPE_COMBO:
 		if(GTK_IS_COMBO(w))
 		{
@@ -2170,6 +2335,9 @@ void PDialogSetPromptCompletePath(gint prompt_num)
 	      case PDIALOG_PROMPT_TYPE_SCALE:
 		break;
 
+	      case PDIALOG_PROMPT_TYPE_POPUP_LIST:
+		break;
+
 	      case PDIALOG_PROMPT_TYPE_COMBO:
 		if(GTK_IS_COMBO(w))
 		{
