@@ -71,7 +71,7 @@ void WinProcessOutputDelete(win_struct *win);
 void WinScanProcessStart(
 	win_struct *win,
 	const gchar *db_location,	/* Virus Scanner Database Location */
-	const gchar *location,		/* Location of object to scan */
+	GList *paths_list,		/* List of paths to scan */
 	const avscan_options options
 );
 void WinScanProcessStop(win_struct *win);
@@ -83,7 +83,7 @@ gboolean WinUpdateProcessIsRunning(win_struct *win);
 /* Scan List & Settings Update */
 void WinScanListOpenFromFile(win_struct *win, const gchar *path);
 void WinScanSettingsUpdate(win_struct *win, const obj_struct *obj);
-void WinResultsLocationUpdate(win_struct *win, const gchar *location);
+void WinResultsLocationUpdate(win_struct *win, GList *paths_list);
 
 /* Operation IDs */
 win_opid_struct *WinOPIDNew(
@@ -120,8 +120,8 @@ void WinStatusMessage(
 void WinStatusTime(
 	win_struct *win, const gulong t, gboolean allow_gtk_iteration
 );
-gboolean WinIsMapped(win_struct *win);          
-void WinMap(win_struct *win);         
+gboolean WinIsMapped(win_struct *win);
+void WinMap(win_struct *win);
 void WinUnmap(win_struct *win);
 void WinDelete(win_struct *win);
 
@@ -166,7 +166,7 @@ void WinProcessOutputDelete(win_struct *win)
 	/* Delete stdout & stderr files */
 	if(win->stdout_path != NULL)      
 	{
-	    unlink(win->stdout_path);  
+	    unlink(win->stdout_path);
 	    g_free(win->stdout_path);
 	    win->stdout_path = NULL;
 	}
@@ -182,18 +182,28 @@ void WinProcessOutputDelete(win_struct *win)
  *	Starts the scan process.
  *
  *	Any current scan process will be stopped first.
+ *
+ *	The db_location specifies the full path to the Virus Scanner
+ *	Database. If this value is NULL then the CFG_PARM_DB_LOCATION
+ *	value from the configuration will be used instead.
+ *
+ *	The paths_list specifies the list of full paths to the locations
+ *	to scan.
+ *
+ *	The options specifies the scan options.
  */
 void WinScanProcessStart(
 	win_struct *win,
 	const gchar *db_location,	/* Virus Scanner Database Location */
-	const gchar *location,		/* Location of object to scan */
+	GList *paths_list,		/* List of paths to scan */
 	const avscan_options options
 )
 {
 	const gulong start_time = (gulong)time(NULL);
 	gint fd;
-	const gchar *tmp_dir;
-	gchar *cmd;
+	const gchar *tmp_dir, *path;
+	gchar *cmd, *cmd_next;
+	GList *glist;
 	const cfg_item_struct *cfg_list;
 	GtkWidget *toplevel;
 	core_struct *core;
@@ -240,7 +250,7 @@ To set the virus database location go to Database->Location...",
 	WinScanProcessStop(win);
 
 	/* Nothing to scan? */
-	if(STRISEMPTY(location))
+	if(paths_list == NULL)
 	{
 #ifdef HAVE_EDV2
 	    EDVPlaySoundWarning(core->edv_ctx);
@@ -259,7 +269,7 @@ To set the virus database location go to Database->Location...",
 	}
 
 	/* Set Results Location */
-	WinResultsLocationUpdate(win, location);
+	WinResultsLocationUpdate(win, paths_list);
 
 	/* Clear the Results List */
 	WinResultsListClear(win);
@@ -300,10 +310,10 @@ To set the virus database location go to Database->Location...",
 	if(fd > -1)
 	    close((int)fd);
 
-	/* Format scan command */
+	/* Format the scan command */
 	cmd = g_strdup_printf(
-	    "\"%s\" --scan \"%s\"%s%s%s --database \"%s\"",
-	    core->prog_path, location,
+	    "\"%s\" --scan%s%s%s --database \"%s\"",
+	    core->prog_path,
 	    (options & AVSCAN_OPT_RECURSIVE) ?
 		" --recursive" : "",
 	    (options & AVSCAN_OPT_EXECUTABLES_ONLY) ?
@@ -312,6 +322,16 @@ To set the virus database location go to Database->Location...",
 		" --ignore-links" : "",
 	    db_location
 	);
+	for(glist = paths_list; glist != NULL; glist = g_list_next(glist))
+	{
+	    path = (const gchar *)glist->data;
+	    if(STRISEMPTY(path))
+		continue;
+
+	    cmd_next = g_strconcat(cmd, " \"", path, "\"", NULL);
+	    g_free(cmd);
+	    cmd = cmd_next;
+	}
 
 	/* Run the scan command */
 	win->scan_pid = (gint)ExecOE(
@@ -363,8 +383,19 @@ To set the virus database location go to Database->Location...",
 }
 
 /*
- *	Interrupts the Win's scan process and deletes stdout and stderr
- *	files.
+ *	Stops the scanning process and cleans up.
+ *
+ *	Stops the scan icon animation.
+ *
+ *	Removes the scan monitoring timeout.
+ *
+ *	Sends SIGINT to the scanning process.
+ *
+ *	Deletes the output files.
+ *
+ *	Resets the status bar and scan values.
+ *
+ *	Marks the Win as no longer passive busy (ready).
  */
 void WinScanProcessStop(win_struct *win)
 {
@@ -382,7 +413,16 @@ void WinScanProcessStop(win_struct *win)
 	    win->scan_toid = 0;
 	}
 
-	/* Interrupt the scan child process */
+	/* Interrupt the scan (child) process
+	 *
+	 * Sending SIGINT to the scan process will increment its stop
+	 * count in the signal callback but it will not exit()
+	 * immediately, instead it will wait for the current (if any)
+	 * ClamAV call to finish and continue to execute without calling
+	 * any additional ClamAV calls except AVScanOPClamAVShutdown()
+	 * in order to shut down ClamAV nicely (so that there are no
+	 * ClamAV tempory files) and finish execution
+	 */
 	if(win->scan_pid > 0)
 	{
 	    kill(win->scan_pid, SIGINT);
@@ -428,6 +468,7 @@ gboolean WinScanProcessIsRunning(win_struct *win)
  */
 void WinUpdateProcessStart(win_struct *win)
 {
+	const gulong start_time = (gulong)time(NULL);
 	gint fd;
 	const gint effective_uid = (gint)geteuid();
 #if defined(HAVE_CLAMAV)
@@ -448,7 +489,7 @@ void WinUpdateProcessStart(win_struct *win)
 
 	toplevel = win->toplevel;
 	core = CORE(win->core);
-	cfg_list = core->cfg_list;             
+	cfg_list = core->cfg_list;
 
 	/* Interrupt virus database update process and delete stdout
 	 * and stderr files
@@ -458,11 +499,11 @@ void WinUpdateProcessStart(win_struct *win)
 	/* Check if the virus update program exists */
 	if(stat(prog, &stat_buf))
 	{
-	    const gint errnum = (gint)errno;
-	    gchar *s, *error_msg = STRDUP(g_strerror(errnum));
+	    const gint error_code = (gint)errno;
+	    gchar *s, *error_msg = STRDUP(g_strerror(error_code));
 	    if(error_msg != NULL)
 	    {
-		*error_msg = toupper(*error_msg);
+		*error_msg = (gchar)toupper((int)*error_msg);
 		s = g_strdup_printf(
 "Unable to find the virus database update program:\n\
 \n\
@@ -525,13 +566,17 @@ Do you want to try and update the virus database anyway?",
 		return;
 	}
 
-	/* Set virus database update monitor timeout callback */
+
+	/* Set the virus database update start time */
+	win->update_start_time = start_time;
+
+	/* Set the virus database update monitor timeout callback */
 	win->update_toid = gtk_timeout_add(
 	    250l,
 	    WinUpdateMonitorTOCB, win
 	);
 
-	/* Get tempory files directory */
+	/* Get the tempory files directory */
 	tmp_dir = g_getenv(ENV_VAR_NAME_TMPDIR);
 	if(STRISEMPTY(tmp_dir))
 #if defined(P_tmpdir)
@@ -542,7 +587,7 @@ Do you want to try and update the virus database anyway?",
 	    tmp_dir = "/tmp";
 #endif
 
-	/* Create output file paths */
+	/* Create the output file paths */
 	win->stdout_path = STRDUP(PrefixPaths(
 	    tmp_dir,
 	    PROG_NAME "XXXXXX"
@@ -560,19 +605,19 @@ Do you want to try and update the virus database anyway?",
 
 	/* Format virus database update command */
 	cmd = g_strdup_printf(
-	    "\"%s\" --verbose",
+	    "\"%s\" --stdout --verbose",
 	    prog
 	);
 
-	/* Run virus database update command */
+	/* Run the virus database update command */
 	win->update_pid = (gint)ExecOE(
 	    cmd, win->stdout_path, win->stderr_path
 	);
 
-	/* Delete command */
+	/* Delete the command */
 	g_free(cmd);
 
-	/* Failed to run virus database update command? */
+	/* Failed to run the virus database update command? */
 	if(win->update_pid <= 0) 
 	{
 	    char *s = g_strdup_printf(
@@ -638,8 +683,9 @@ void WinUpdateProcessStop(win_struct *win)
 
 	/* Reset stop count */
 	win->stop_count = 0;
+	win->update_start_time = 0l;
 
-	/* Reset progress and time */
+	/* Reset the progress and time */
 	WinStatusProgress(win, 0.0f, FALSE);
 	WinStatusTime(win, 0l, FALSE);
 
@@ -653,7 +699,7 @@ void WinUpdateProcessStop(win_struct *win)
  */
 gboolean WinUpdateProcessIsRunning(win_struct *win)
 {
-	gint p = (win != NULL) ? win->update_pid : 0;
+	const gint p = (win != NULL) ? win->update_pid : 0;
 	return((p > 0) ? ExecProcessExists((pid_t)p) : FALSE);
 }
 
@@ -694,7 +740,7 @@ void WinScanListOpenFromFile(win_struct *win, const gchar *path)
 	    WinScanListAppend(win, OBJ(glist->data));
 
 	/* Delete scan list */
-	g_list_free(obj_list);                         
+	g_list_free(obj_list);
 
 	gtk_clist_thaw(clist);
 
@@ -765,20 +811,43 @@ void WinScanSettingsUpdate(win_struct *win, const obj_struct *obj)
 /*
  *	Updates the Win's results location.
  */
-void WinResultsLocationUpdate(win_struct *win, const gchar *location)
+void WinResultsLocationUpdate(win_struct *win, GList *paths_list)
 {
 	GtkWidget *w;
 
 	if(win == NULL)
-	    return;  
+	    return;
 
 	win->freeze_count++;
 
 	w = win->results_location_label;
 	if(w != NULL)
-	    gtk_label_set_text(
-		GTK_LABEL(w), (location != NULL) ? location : ""
-	    );
+	{
+	    gchar *s;
+	    if(paths_list != NULL)
+	    {
+		const gint npaths = g_list_length(paths_list);
+		if(npaths == 1)
+		{
+		    s = STRDUP((const gchar *)paths_list->data);
+		}
+		else
+		{
+		    GList *glist = g_list_last(paths_list);
+		    s = g_strdup_printf(
+			"..., %s (%i locations)",
+			(glist != NULL) ? (const gchar *)glist->data : "",
+			npaths
+		    );
+		}
+	    }
+	    else
+	    {
+		s = STRDUP("");
+	    }
+	    gtk_label_set_text(GTK_LABEL(w), s);
+	    g_free(s);
+	}
 
 	win->freeze_count--;
 
@@ -897,7 +966,7 @@ GList *WinOPIDListNew(win_struct *win)
 	accel_key = 0;
 	accel_mods = 0;
 	button_icon_data = (guint8 **)icon_add_20x20_xpm;
-	menu_icon_data = button_icon_data;                 
+	menu_icon_data = button_icon_data;
 	func_cb = WinAddCB;
 	DO_APPEND(WIN_OPID_ADD)
 
@@ -1035,7 +1104,7 @@ GList *WinOPIDListNew(win_struct *win)
 
 
 	allow_multiple = FALSE;
-	item_type = TOOLBAR_ITEM_BUTTON;  
+	item_type = TOOLBAR_ITEM_BUTTON;
 	button_name = "Start";
 	menu_name = button_name;
 	tooltip = "Start scanning";
@@ -1047,7 +1116,7 @@ GList *WinOPIDListNew(win_struct *win)
 	DO_APPEND(WIN_OPID_START)
 
 	allow_multiple = FALSE;
-	item_type = TOOLBAR_ITEM_BUTTON;  
+	item_type = TOOLBAR_ITEM_BUTTON;
 	button_name = "Stop";
 	menu_name = button_name;
 	tooltip = "Stop the current operation";
@@ -1058,7 +1127,7 @@ GList *WinOPIDListNew(win_struct *win)
 	func_cb = WinStopCB;
 	DO_APPEND(WIN_OPID_STOP)
 
-	allow_multiple = FALSE;   
+	allow_multiple = FALSE;
 	item_type = TOOLBAR_ITEM_BUTTON;
 	button_name = "Location";
 	menu_name = "Location...";
@@ -1090,7 +1159,7 @@ GList *WinOPIDListNew(win_struct *win)
 	accel_key = 0;
 	accel_mods = 0;
 	button_icon_data = NULL;
-	menu_icon_data = button_icon_data;                         
+	menu_icon_data = button_icon_data;
 	func_cb = WinExecutablesOnlyCB;
 	DO_APPEND(WIN_OPID_EXECUTABLES_ONLY)
 
@@ -1102,7 +1171,7 @@ GList *WinOPIDListNew(win_struct *win)
 	accel_key = 0;
 	accel_mods = 0;
 	button_icon_data = NULL;
-	menu_icon_data = button_icon_data;                         
+	menu_icon_data = button_icon_data;
 	func_cb = WinIgnoreLinksCB;
 	DO_APPEND(WIN_OPID_IGNORE_LINKS)
 
@@ -1145,8 +1214,8 @@ GList *WinOPIDListNew(win_struct *win)
 
 	allow_multiple = FALSE;
 	item_type = TOOLBAR_ITEM_BUTTON;
-	button_name = "UnselAll";       
-	menu_name = "Unselect All";  
+	button_name = "UnselAll";
+	menu_name = "Unselect All";
 	tooltip = "Unselect all objects";
 	accel_key = 0;
 	accel_mods = 0;
@@ -1164,7 +1233,7 @@ GList *WinOPIDListNew(win_struct *win)
 	accel_mods = 0;
 	button_icon_data = (guint8 **)icon_select_20x20_xpm; 
 	menu_icon_data = NULL;
-	func_cb = WinResultsInvertSelectionCB;  
+	func_cb = WinResultsInvertSelectionCB;
 	DO_APPEND(WIN_OPID_RESULTS_INVERT_SELECTION)
 
 	allow_multiple = FALSE;
@@ -1180,7 +1249,7 @@ GList *WinOPIDListNew(win_struct *win)
 	DO_APPEND(WIN_OPID_RESULTS_OPEN)
 
 	allow_multiple = FALSE;
-	item_type = TOOLBAR_ITEM_BUTTON;       
+	item_type = TOOLBAR_ITEM_BUTTON;
 	button_name = "Save As";
 	menu_name = "Save As...";
 	tooltip = "Save scan results log as";
@@ -1330,8 +1399,8 @@ GList *WinOPIDListNew(win_struct *win)
 	button_name = "Update?";
 	menu_name = "How To Update";
 	tooltip = "Help on updating the virus database";
-	accel_key = 0;                      
-	accel_mods = 0;    
+	accel_key = 0;
+	accel_mods = 0;
 	button_icon_data = (guint8 **)icon_help_20x20_xpm;
 	menu_icon_data = NULL;
 	func_cb = WinHelpUpdatingCB;
@@ -1414,7 +1483,7 @@ static void WinBuildMenuBar(win_struct *win)
   win->opid, (_id_)				\
  );						\
  if(opid != NULL) {				\
-  w = GUIMenuItemCreate(                        \
+  w = GUIMenuItemCreate(			\
    menu, GUI_MENU_ITEM_TYPE_LABEL, accelgrp,	\
    (icon_data != NULL) ?			\
     icon_data : opid->menu_icon_data,		\
@@ -1422,12 +1491,12 @@ static void WinBuildMenuBar(win_struct *win)
    opid->accel_key, opid->accel_mods,		\
    (gpointer *)&menuitem_func_w,		\
    opid->win, opid->func_cb			\
-  );                                            \
-  GUISetMenuItemCrossingCB(                     \
-   w,                                           \
+  );						\
+  GUISetMenuItemCrossingCB(			\
+   w,						\
    (gpointer)opid->enter_func_cb, opid,		\
    (gpointer)opid->leave_func_cb, opid		\
-  );                                            \
+  );						\
  }						\
  else						\
   w = NULL;					\
@@ -1454,15 +1523,15 @@ static void WinBuildMenuBar(win_struct *win)
   );						\
  }						\
  else                                           \
-  w = NULL;                                     \
+  w = NULL;					\
 }
 
 #define DO_ADD_MENU_SEP		{		\
  w = GUIMenuItemCreate(				\
-  menu, GUI_MENU_ITEM_TYPE_SEPARATOR, NULL,     \
-  NULL, NULL, 0, 0, NULL,                       \
+  menu, GUI_MENU_ITEM_TYPE_SEPARATOR, NULL,	\
+  NULL, NULL, 0, 0, NULL,			\
   NULL, NULL                                    \
- );                                             \
+ );						\
 }
 
 	/* Create the File menu */
@@ -1558,6 +1627,55 @@ static void WinBuildMenuBar(win_struct *win)
 	    , GUI_MENU_BAR_ALIGN_LEFT
 	);
 
+
+	/* Create the View menu */
+	menu = GUIMenuCreateTearOff();
+	if(menu != NULL)
+	{
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_SCAN_PAGE)
+	    win->view_scan_page_micheck = w;
+
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_RESULTS_PAGE)
+	    win->view_results_page_micheck = w;
+
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_DB_PAGE)
+	    win->view_db_page_micheck = w;
+
+	    DO_ADD_MENU_SEP
+
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_TOOLBAR)
+	    win->view_toolbar_micheck = w;
+
+	    icon_data = NULL;
+	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_STATUSBAR)
+	    win->view_statusbar_micheck = w;
+	}
+	win->view_mlabel = GUIMenuAddToMenuBar( 
+	    menubar, menu,
+#if defined(PROG_LANGUAGE_SPANISH)
+"Vista"
+#elif defined(PROG_LANGUAGE_FRENCH)
+"Vue"
+#elif defined(PROG_LANGUAGE_GERMAN)
+"Blick"
+#elif defined(PROG_LANGUAGE_ITALIAN)
+"Veduta"
+#elif defined(PROG_LANGUAGE_DUTCH)
+"Overzicht"
+#elif defined(PROG_LANGUAGE_PORTUGUESE)
+"Vista"
+#elif defined(PROG_LANGUAGE_NORWEGIAN)
+"Sikt"
+#else
+"View"
+#endif
+	    , GUI_MENU_BAR_ALIGN_LEFT
+	);
+
 	/* Create the Scan menu */
 	menu = GUIMenuCreateTearOff();
 	if(menu != NULL)
@@ -1660,55 +1778,6 @@ static void WinBuildMenuBar(win_struct *win)
 	);
 
 
-	/* Create the View menu */
-	menu = GUIMenuCreateTearOff();
-	if(menu != NULL)
-	{
-	    icon_data = NULL;
-	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_SCAN_PAGE)
-	    win->view_scan_page_micheck = w;
-
-	    icon_data = NULL;
-	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_RESULTS_PAGE)
-	    win->view_results_page_micheck = w;
-
-	    icon_data = NULL;
-	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_DB_PAGE)
-	    win->view_db_page_micheck = w;
-
-	    DO_ADD_MENU_SEP
-
-	    icon_data = NULL;
-	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_TOOLBAR)
-	    win->view_toolbar_micheck = w;
-
-	    icon_data = NULL;
-	    DO_ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_STATUSBAR)
-	    win->view_statusbar_micheck = w;
-	}
-	win->view_mlabel = GUIMenuAddToMenuBar( 
-	    menubar, menu,
-#if defined(PROG_LANGUAGE_SPANISH)
-"Vista"
-#elif defined(PROG_LANGUAGE_FRENCH)
-"Vue"
-#elif defined(PROG_LANGUAGE_GERMAN)
-"Blick"
-#elif defined(PROG_LANGUAGE_ITALIAN)
-"Veduta"
-#elif defined(PROG_LANGUAGE_DUTCH)
-"Overzicht"
-#elif defined(PROG_LANGUAGE_PORTUGUESE)
-"Vista"
-#elif defined(PROG_LANGUAGE_NORWEGIAN)
-"Sikt"
-#else
-"View"
-#endif
-	    , GUI_MENU_BAR_ALIGN_LEFT
-	);
-
-
 	/* Create the Help menu */
 	menu = GUIMenuCreateTearOff();
 	if(menu != NULL)
@@ -1979,7 +2048,7 @@ static void WinBuildLists(win_struct *win)
 	/* Left column vbox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
-	gtk_widget_show(w);                                   
+	gtk_widget_show(w);
 	parent2 = w;
 #endif
 	/* Left column notebook */
@@ -1987,7 +2056,7 @@ static void WinBuildLists(win_struct *win)
 	notebook = GTK_NOTEBOOK(w);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0); 
 	gtk_notebook_set_tab_pos(notebook, GTK_POS_TOP);
-	gtk_notebook_set_scrollable(notebook, TRUE);   
+	gtk_notebook_set_scrollable(notebook, TRUE);
 	gtk_notebook_set_show_tabs(notebook, TRUE); 
 	gtk_notebook_set_show_border(notebook, TRUE);
 	gtk_signal_connect(
@@ -2142,7 +2211,7 @@ static void WinBuildLists(win_struct *win)
 	win->add_btn = w;
 	NEW_BUTTON(WIN_OPID_EDIT, parent5, FALSE);
 	win->edit_btn = w;
-	NEW_BUTTON(WIN_OPID_REMOVE, parent5, FALSE);   
+	NEW_BUTTON(WIN_OPID_REMOVE, parent5, FALSE);
 	win->remove_btn = w;
 
 
@@ -2199,7 +2268,8 @@ static void WinBuildLists(win_struct *win)
 	);
 	SET_CROSSING(WIN_OPID_LOCATION, w);
 	GUISetWidgetTip(w,
-"Enter the location of the file or directory to scan"
+"Enter a comma-separated list of full paths to the file(s)\
+ and/or directory(ies) to scan"
 	);
 
 	/* Scan settings hbox */
@@ -2348,7 +2418,7 @@ static void WinBuildLists(win_struct *win)
 	gtk_clist_set_column_resizeable(clist, 1, FALSE);
 	gtk_clist_set_column_width(clist, 1, 10);
 	gtk_clist_set_row_height(clist, WIN_LIST_ROW_SPACING);
-	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);   
+	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "resize_column",
 	    GTK_SIGNAL_FUNC(WinListResizeColumnCB), win
@@ -2393,7 +2463,7 @@ static void WinBuildLists(win_struct *win)
 
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_end(GTK_BOX(parent5), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);   
+	gtk_widget_show(w);
 	parent5 = w;
 	NEW_BUTTON(WIN_OPID_RESULTS_CLEAR, parent5, FALSE);
 	win->results_clear_btn = w;
@@ -2414,7 +2484,7 @@ static void WinBuildLists(win_struct *win)
 	/* Virus Database page */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
-	gtk_notebook_append_page(                                 
+	gtk_notebook_append_page(
 	    GTK_NOTEBOOK(parent2), w, tablabel
 	);
 	gtk_widget_show(w);
@@ -2436,8 +2506,8 @@ static void WinBuildLists(win_struct *win)
 	/* Virus Database Stats & Buttons hbox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);                                      
-	parent5 = w;       
+	gtk_widget_show(w);
+	parent5 = w;
 
 
 	/* Virus Database Stats Prefixes icon & label */
@@ -2450,7 +2520,7 @@ static void WinBuildLists(win_struct *win)
 	w = gtk_label_new("Location:\nLast Updated:\nPatterns:");
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);                                      
+	gtk_widget_show(w);
 
 	/* Virus Database Stats label */
 	win->db_stats_label = w = gtk_label_new("");
@@ -2573,7 +2643,27 @@ static void WinBuildLists(win_struct *win)
 	parent3 = w;
 	/* Animated Icon */
 	win->anim_icon = w = WinAnimIconNew();
+	gtk_widget_add_events(
+	    w,
+	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+	    GDK_POINTER_MOTION_MASK |
+	    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+	);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+	GUISetWidgetTip(
+	    w,
+"Drag an object here to scan it for viruses"
+	);
+	GUIDNDSetTar(
+	    w,
+	    dnd_tar_types,
+	    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
+	    GDK_ACTION_COPY,			/* Actions */
+	    GDK_ACTION_COPY,			/* Default action if same */
+	    GDK_ACTION_COPY,			/* Default action */
+	    WinAnimIconDataReceivedCB,
+	    win
+	);
 	gtk_widget_show(w);
 
 
@@ -2738,15 +2828,15 @@ static void WinBuildListsMenu(win_struct *win)
    opid->accel_key, opid->accel_mods,		\
    (gpointer *)&menuitem_func_w,		\
    opid->win, opid->func_cb			\
-  );                                            \
-  GUISetMenuItemCrossingCB(                     \
-   w,                                           \
-   (gpointer)opid->enter_func_cb, opid,         \
-   (gpointer)opid->leave_func_cb, opid          \
-  );                                            \
+  );						\
+  GUISetMenuItemCrossingCB(			\
+   w,						\
+   (gpointer)opid->enter_func_cb, opid,		\
+   (gpointer)opid->leave_func_cb, opid		\
+  );						\
  }						\
  else                                           \
-  w = NULL;                                     \
+  w = NULL;					\
 }
 
 #define DO_ADD_MENU_ITEM_CHECK(_id_)	{	\
@@ -2762,15 +2852,15 @@ static void WinBuildListsMenu(win_struct *win)
    opid->accel_key, opid->accel_mods,		\
    (gpointer *)&menuitem_func_w,		\
    opid->win, opid->func_cb			\
-  );                                            \
-  GUISetMenuItemCrossingCB(                     \
-   w,                                           \
-   (gpointer)opid->enter_func_cb, opid,         \
-   (gpointer)opid->leave_func_cb, opid          \
-  );                                            \
+  );						\
+  GUISetMenuItemCrossingCB(			\
+   w,						\
+   (gpointer)opid->enter_func_cb, opid,		\
+   (gpointer)opid->leave_func_cb, opid		\
+  );						\
  }						\
  else                                           \
-  w = NULL;                                     \
+  w = NULL;					\
 }
 
 #define DO_ADD_MENU_SEP		{		\
@@ -2884,7 +2974,7 @@ win_struct *WinNew(gpointer core_ptr)
 
 	cfg_list = core->cfg_list;
 
-	win->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);  
+	win->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	win->accelgrp = accelgrp = gtk_accel_group_new();
 	win->processing = FALSE;
 	win->busy_count = 0;
@@ -2919,6 +3009,7 @@ win_struct *WinNew(gpointer core_ptr)
 	w = toplevel;
 	gtk_widget_add_events(
 	    w,
+	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK
 	);
 	gtk_signal_connect(
@@ -2963,7 +3054,7 @@ win_struct *WinNew(gpointer core_ptr)
 
 	/* Main VBox */
 	win->main_vbox = main_vbox = w = gtk_vbox_new(FALSE, 0);
-	gtk_container_border_width(GTK_CONTAINER(w), 0);                        
+	gtk_container_border_width(GTK_CONTAINER(w), 0);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	gtk_widget_show(w);
 	parent = w;
@@ -3063,7 +3154,7 @@ void WinSavePositions(win_struct *win)
 	    CFGItemListSetValueI(
 		cfg_list, CFG_PARM_WIN_X,
 		x, FALSE
-	    );          
+	    );
 	    CFGItemListSetValueI(
 		cfg_list, CFG_PARM_WIN_Y,
 		y, FALSE
@@ -3178,7 +3269,7 @@ void WinUpdate(win_struct *win)
 	);
 	ToolBarItemSetSensitiveID(
 	    toolbar, WIN_OPID_ADD, sensitive
-	);      
+	);
 
 	/* Close */
 	sensitive = (scanning || db_loading || db_updating) ? FALSE : TRUE;
@@ -3627,7 +3718,7 @@ void WinStatusMessage(
 
 	gtk_label_set_text(
 	    GTK_LABEL(w), (mesg != NULL) ? mesg : ""
-	);  
+	);
 
 	while((gtk_events_pending() > 0) && allow_gtk_iteration)
 	    gtk_main_iteration();
@@ -3654,7 +3745,7 @@ void WinStatusTime(
 	    );
 	else
 	    s = g_strdup_printf(
-		"%.2i:%.2i",
+		"%i:%.2i",
 		(gint)(t / 60l),
 		(gint)(t % 60l)
 	    );
