@@ -202,7 +202,7 @@ static const gchar *GET_NAME_FROM_PATH(const gchar *path)
 	if(STRISEMPTY(path))
 	    return(path);
 			 
-	s = strrchr(path, DIR_DELIMINATOR);
+	s = strrchr(path, G_DIR_SEPARATOR);
 	return((s != NULL) ? (s + 1) : path);
 }
 
@@ -290,7 +290,7 @@ gint WinEventCB(
 	    press = (event->type == GDK_KEY_PRESS) ? TRUE : FALSE;
 	    keyval = key->keyval;
 	    state = key->state;
-#define DO_STOP_KEY_SIGNAL_EMIT {               \
+#define DO_STOP_KEY_SIGNAL_EMIT {		\
  gtk_signal_emit_stop_by_name(			\
   GTK_OBJECT(widget),				\
   press ?                                       \
@@ -424,17 +424,17 @@ void WinNotebookSwitchPageCB(
  *	signal callback.
  */
 gint WinEntryCompletePathKeyCB(
-        GtkWidget *widget, GdkEventKey *key, gpointer data
+	GtkWidget *widget, GdkEventKey *key, gpointer data
 )
 {
-        gint status = FALSE;
-        gboolean press;
-        GtkEntry *entry = GTK_ENTRY(widget);
+	gint status = FALSE;
+	gboolean press;
+	GtkEntry *entry = GTK_ENTRY(widget);
 	win_struct *win = WIN(data);
-        if((entry == NULL) || (key == NULL) || (win == NULL))
-            return(status);
+	if((entry == NULL) || (key == NULL) || (win == NULL))
+	    return(status);
 
-        press = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
+	press = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
 
 #define SIGNAL_EMIT_STOP	{		\
  gtk_signal_emit_stop_by_name(			\
@@ -444,38 +444,38 @@ gint WinEntryCompletePathKeyCB(
  );						\
 }
 
-        switch(key->keyval)
-        {
-          case GDK_Tab:
-            if(press)
-            {
-                gchar *s = STRDUP(gtk_entry_get_text(entry));
-                if(s != NULL)
-                {
-                    gint status;
+	switch(key->keyval)
+	{
+	  case GDK_Tab:
+	    if(press)
+	    {
+		gchar *s = STRDUP(gtk_entry_get_text(entry));
+		if(s != NULL)
+		{
+		    gint status;
 
-                    s = CompletePath(s, &status);
-                    gtk_entry_set_text(entry, (s != NULL) ? s : "");
-                    gtk_entry_set_position(entry, -1);
-                    g_free(s);
+		    s = CompletePath(s, &status);
+		    gtk_entry_set_text(entry, (s != NULL) ? s : "");
+		    gtk_entry_set_position(entry, -1);
+		    g_free(s);
 
-                    switch(status)
-                    {
-                      case COMPLETE_PATH_NONE:
-                      case COMPLETE_PATH_AMBIGUOUS:
-                        gdk_beep();
-                        break;
-                    }
-                }
-            }
-            SIGNAL_EMIT_STOP
-            status = TRUE;
-            break;
-        }
+		    switch(status)
+		    {
+		      case COMPLETE_PATH_NONE:
+		      case COMPLETE_PATH_AMBIGUOUS:
+			gdk_beep();
+			break;
+		    }
+		}
+	    }
+	    SIGNAL_EMIT_STOP
+	    status = TRUE;
+	    break;
+	}
 
 #undef SIGNAL_EMIT_STOP
 
-        return(status);
+	return(status);
 }
 
 
@@ -502,7 +502,7 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    press = (event->type == GDK_KEY_PRESS) ? TRUE : FALSE;
 	    keyval = key->keyval;
 	    state = key->state;
-#define DO_STOP_KEY_SIGNAL_EMIT {               \
+#define DO_STOP_KEY_SIGNAL_EMIT {		\
  gtk_signal_emit_stop_by_name(			\
   GTK_OBJECT(widget),				\
   press ?                                       \
@@ -573,7 +573,7 @@ gint WinListEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    {
 	      case 1:
 		if(widget == win->scan_clist)
-		{  
+		{
 		    WinStartCB(NULL, win);
 		}
 		break;
@@ -862,7 +862,7 @@ void WinTreeSelectRowCB(
 
 /*
  *      Win GtkCTree "tree_unselect_row" signal callback.
- */                       
+ */
 void WinTreeUnselectRowCB(
 	GtkCTree *ctree, GtkCTreeNode *node, gint column,
 	gpointer data                                    
@@ -891,7 +891,7 @@ void WinTreeUnselectRowCB(
 void WinTreeExpandCB(
 	GtkCTree *ctree, GtkCTreeNode *node, gpointer data
 )
-{  
+{
 	win_struct *win = WIN(data);
 	if((ctree == NULL) || (win == NULL))
 	    return;
@@ -1116,14 +1116,33 @@ gint WinScanMonitorTOCB(gpointer data)
 	clist = (GtkCList *)win->results_clist;
 	core = CORE(win->core);
 
-	/* Need to stop? */
+	/* User clicked on the stop button? */
 	if(win->stop_count > 0)
 	{
-	    /* Stop scanning */
+	    gint new_row;
+
+	    /* Stop scanning
+	     *
+	     * Mark that this timeout is no longer being called
+	     */
 	    win->scan_toid = 0;
+
+	    new_row = WinResultsListAppend(win, NULL, "Interrupted");
+	    if(new_row > -1)
+	    {
+		gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
+	    }
+
+	    /* Tell the scanning process to stop scanning, delete
+	     * all the tempory output files, and clean up
+	     */
 	    WinScanProcessStop(win);
 
-	    WinStatusMessage(win, "Scan interrupted", FALSE);
+	    WinStatusMessage(
+		win,
+"Scan interrupted",
+		FALSE
+	    );
 
 	    return(FALSE);
 	}
@@ -1146,24 +1165,23 @@ gint WinScanMonitorTOCB(gpointer data)
 	     *
 	     *     The virus engine is out of date, upgrade!
 	     */
-	    gint len, new_row;
-	    gchar *s, *s2, *path, *virus_name;
+	    gint new_row;
+	    gchar *s_end, *path, *virus_name;
+	    const gchar *s;
 
 	    while(fgets(buf, sizeof(buf), fp) != NULL)
 	    {
 		buf[sizeof(buf) - 1] = '\0';
 
-		s = strpbrk(buf, "\n\r");
-		if(s != NULL)
-		    *s = '\0';
+		s_end = strpbrk(buf, "\n\r");
+		if(s_end != NULL)
+		    *s_end = '\0';
 
 		path = NULL;			/* Path */
 		virus_name = NULL;		/* Virus|problem */
 
 		/* Seek past initial spaces */
-		s = buf;
-		while(ISBLANK(*s))
-		    s++;
+		s = SEEK_PAST_SPACES(buf);
 
 		/* Check format, if the first character is a "
 		 * then it implies the format is:
@@ -1174,41 +1192,28 @@ gint WinScanMonitorTOCB(gpointer data)
 		{
 		    s++;	/* Seek past initial '"' character */
 
-		    /* Seek s2 to next '"' character (ignore any escape
-		     * characters) or end of string
+		    /* Get the path and increment s past the next '"'
+		     * character
 		     */
-		    for(s2 = s; *s2 != '\0'; s2++)
-		    {
-			if(*s2 == '"')
-			    break;
-		    }
-		    /* Calculate length and copy path */
-		    len = (gint)(s2 - s);
-		    if(len > 0)
-		    {
-			path = (gchar *)g_malloc((len + 1) * sizeof(gchar));
-			if(path != NULL)
-			{
-			    memcpy(path, s, len * sizeof(gchar));
-			    path[len] = '\0';
-			}
-		    }
+		    s = GET_ARG(s, '"', &path);
 
-		    /* Seek s to the start of virus|problem */
-		    s = (*s2 == '"') ? (s2 + 1) : s2;
-		    while(ISBLANK(*s))
-			s++;
+		    s = SEEK_PAST_SPACES(s);
+
+		    /* Get the virus/problem */
 		    virus_name = STRDUP(s);
 		}
 		else
 		{
+		    /* No path, just get the whole line as the virus
+		     * name
+		     */
 		    virus_name = STRDUP(s);
 		}
 
 		new_row = WinResultsListAppend(win, path, virus_name);
 		if(new_row > -1)
 		{
-		    gtk_clist_moveto(clist, new_row, 0, 0.0f, 1.0f);
+		    gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
 		}
 
 		g_free(path);
@@ -1450,7 +1455,11 @@ gint WinScanMonitorTOCB(gpointer data)
 	    win->scan_toid = 0;
 	    WinScanProcessStop(win);
 
-	    WinStatusMessage(win, "Scan done", TRUE);
+	    WinStatusMessage(
+		win,
+"Scan done",
+		FALSE
+	    );
 
 	    return(FALSE);
 	}
@@ -1535,7 +1544,9 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	fp = win->stderr_fp;
 	if(fp != NULL)
 	{
+	    gint new_row;
 	    gchar *s;
+	    GtkCList *clist = GTK_CLIST(win->results_clist);
 
 	    while(fgets(buf, sizeof(buf), fp) != NULL)
 	    {
@@ -1551,14 +1562,23 @@ gint WinUpdateMonitorTOCB(gpointer data)
 		while(ISBLANK(*s))
 		    s++;
 
-		WinStatusMessage(win, s, TRUE);
+		new_row = WinResultsListAppend(
+		    win,
+		    NULL,			/* No path */
+		    s				/* Virus/Problem */
+		);
+		if(new_row > -1)
+		{
+		    gtk_clist_moveto(clist, new_row, 0, 1.0f, 0.0f);
+		}
 	    }
 	}
 
+
 	/* Process has exited? */ 
 	if(!WinUpdateProcessIsRunning(win))
 	{
-	    /* Mark virus database update time */
+	    /* Mark the virus database update time */
 	    CFGItemListSetValueL(
 		core->cfg_list, CFG_PARM_DB_LAST_UPDATE,
 		(gulong)time(NULL), FALSE
@@ -1569,7 +1589,7 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	    EDVPlaySoundCompleted(core->edv_ctx);
 #endif
 
-	    /* Stop virus database update */
+	    /* Stop the virus database update */
 	    win->update_toid = 0;
 	    WinUpdateProcessStop(win);
 
@@ -1593,8 +1613,19 @@ gint WinUpdateMonitorTOCB(gpointer data)
 	    return(FALSE);
 	}
 
-	/* Update progress */
+	/* Update the progress */
 	WinStatusProgress(win, -1.0f, FALSE);
 
+	/* Update the time */
+	if(win->update_start_time > 0l)
+	{
+	    const gulong t = (gulong)time(NULL);
+	    WinStatusTime(
+		win,
+		(t > win->update_start_time) ? (t - win->update_start_time) : 0l,
+		FALSE
+	    );
+	}
+
 	return(TRUE);
 }
