@@ -33,12 +33,12 @@ static void WinDBUpdateIterate(
 	core_struct *core, win_struct *win,
 	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
 	struct cl_node *clam_pnode,
-	gboolean verbose, gint *patterns_loaded
+	const gboolean verbose, gint *patterns_loaded
 );
 #endif
 void WinDBUpdate(
 	win_struct *win, const gchar *db_location,
-	gboolean verbose
+	const gboolean verbose
 );
 void WinDBQueueUpdate(win_struct *win);
 void WinDBClear(win_struct *win);
@@ -229,7 +229,7 @@ static void WinDBUpdateIterate(
 	core_struct *core, win_struct *win,
 	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
 	struct cl_node *clam_pnode,
-	gboolean verbose, gint *patterns_loaded
+	const gboolean verbose, gint *patterns_loaded
 )
 {
 	gint i;
@@ -475,7 +475,7 @@ static void WinDBUpdateIterate(
  */
 void WinDBUpdate(
 	win_struct *win, const gchar *db_location,
-	gboolean verbose
+	const gboolean verbose
 )
 {
 #if defined(HAVE_CLAMAV)
@@ -541,13 +541,13 @@ void WinDBUpdate(
 #if defined(HAVE_CLAMAV)
 	if(stat(db_location, &stat_buf))
 	{
-	    const gint errnum = errno;
+	    const gint error_code = (gint)errno;
 	    if(verbose)
 	    {
-		gchar *s, *error_msg = STRDUP(g_strerror(errnum));
+		gchar *s, *error_msg = STRDUP(g_strerror(error_code));
 		if(error_msg != NULL)
 		{
-		    *error_msg = toupper(*error_msg);
+		    *error_msg = (gchar)toupper((int)*error_msg);
 		    s = g_strdup_printf(
 			"%s: %s",
 			db_location,
