@@ -7,7 +7,7 @@
 
 extern void WinDBUpdate(
 	win_struct *win, const gchar *db_location,
-	gboolean verbose
+	const gboolean verbose
 );
 extern void WinDBQueueUpdate(win_struct *win);
 extern void WinDBClear(win_struct *win);
