@@ -4,6 +4,7 @@
 #include "cdialog.h"
 #include "obj.h"
 #include "win.h"
+#include "winopcb.h"
 #include "winlist.h"
 #include "windnd.h"
 #include "core.h"
@@ -16,6 +17,13 @@ static void WinDragDataReceivedNexus(
 	gint row, gint column
 );
 
+void WinAnimIconDataReceivedCB(
+        GtkWidget *widget, GdkDragContext *dc,
+        gint x, gint y,
+        GtkSelectionData *selection_data, guint info, guint t,
+        gpointer data
+);
+
 void WinListDNDSetIcon(
 	GtkCList *clist, gint row, gint column
 );
@@ -63,6 +71,7 @@ static void WinDragDataReceivedNexus(
 	gint row, gint column
 )
 {
+	core_struct *core = win->core;
 	GList *obj_list = ObjDDEBufferParse(
 	    (const guint8 *)selection_data->data,
 	    selection_data->length
@@ -70,10 +79,12 @@ static void WinDragDataReceivedNexus(
 	const gint nobjs = g_list_length(obj_list);
 
 #define FREE_ALL		{	\
- g_list_foreach(			\
-  obj_list, (GFunc)ObjDelete, NULL	\
- );					\
- g_list_free(obj_list);			\
+ if(obj_list != NULL) {			\
+  g_list_foreach(			\
+   obj_list, (GFunc)ObjDelete, NULL	\
+  );					\
+  g_list_free(obj_list);		\
+ }					\
 }
 
 	if((obj_list == NULL) || (nobjs <= 0))
@@ -90,8 +101,70 @@ static void WinDragDataReceivedNexus(
 	   (info == DND_INFO_STRING)
 	)
 	{
+	    /* Animation Icon */
+	    if(w == win->anim_icon)
+	    {
+		gchar *s, *location;
+		GList *glist;
+		GtkWidget *toplevel = win->toplevel;
+		GtkEntry *entry = GTK_ENTRY(win->location_entry);
+		obj_struct *obj;
+
+		/* Is there a scan or update already in progress? */
+		if((win->scan_toid != 0) || (win->update_toid != 0))
+		{
+#ifdef HAVE_EDV2
+		    EDVPlaySoundWarning(core->edv_ctx);
+#endif
+		    CDialogSetTransientFor(toplevel);
+		    CDialogGetResponse(
+			"Scan Failed",
+"A scan or virus database update is already in progress.\n\
+\n\
+You may either wait for the current scan or update to\n\
+complete itself or click on the stop button to stop the\n\
+current scan or update before starting a new scan.",
+			NULL,
+			CDIALOG_ICON_WARNING,
+			CDIALOG_BTNFLAG_OK,
+			CDIALOG_BTNFLAG_OK
+		    );
+		    CDialogSetTransientFor(NULL);
+		    FREE_ALL
+		    WinSetBusy(win, FALSE);
+		    return;
+		}
+
+		location = STRDUP("");
+		for(glist = obj_list;
+		    glist != NULL;
+		    glist = g_list_next(glist)
+		)
+		{
+		    obj = OBJ(glist->data);
+		    if(obj == NULL)
+			continue;
+
+		    s = g_strconcat(
+			location,
+			obj->path,
+			(g_list_next(glist) != NULL) ? "," : "",
+			NULL
+		    );
+		    g_free(location);
+		    location = s;
+		}
+
+		gtk_entry_set_text(entry, location);
+		gtk_entry_set_position(entry, -1);
+
+		g_free(location);
+
+		/* Start scan */
+		WinStartCB(NULL, win);
+	    }
 	    /* Scan List? */
-	    if(w == win->scan_clist)
+	    else if(w == win->scan_clist)
 	    {
 		GList *glist;
 		GtkCList *clist = GTK_CLIST(w);
@@ -136,17 +209,34 @@ static void WinDragDataReceivedNexus(
 	    /* Location Entry */
 	    else if(w == win->location_entry)
 	    {
-		GList *glist = obj_list;
-		const obj_struct *obj = OBJ((glist != NULL) ?
-		    glist->data : NULL
-		);
+		gchar *s, *location = STRDUP("");
+		GList *glist;
 		GtkEntry *entry = GTK_ENTRY(w);
-		gtk_entry_set_text(
-		    entry,
-		    (obj != NULL) ?
-			((obj->path != NULL) ? obj->path : "") : ""
-		);
+		obj_struct *obj;
+
+		for(glist = obj_list;
+		    glist != NULL;
+		    glist = g_list_next(glist)
+		)
+		{
+		    obj = OBJ(glist->data);
+		    if(obj == NULL)
+			continue;
+
+		    s = g_strconcat(
+			location,
+			obj->path,
+			(g_list_next(glist) != NULL) ? "," : "",
+			NULL
+		    );
+		    g_free(location);
+		    location = s;
+		}
+
+		gtk_entry_set_text(entry, location);
 		gtk_entry_set_position(entry, -1);
+
+		g_free(location);
 	    }
 	    /* Results List */
 	    else if(w == win->results_clist)
@@ -157,13 +247,34 @@ static void WinDragDataReceivedNexus(
 
 	WinUpdate(win);
 
-	WinSetBusy(win, FALSE);
-
 	FREE_ALL
+
+	WinSetBusy(win, FALSE);
 #undef FREE_ALL
 }
 
 
+/*
+ *	Animation Icon "drag_data_received" signal callback.
+ */
+void WinAnimIconDataReceivedCB(
+        GtkWidget *widget, GdkDragContext *dc,
+        gint x, gint y,
+        GtkSelectionData *selection_data, guint info, guint t,
+        gpointer data
+)
+{
+	win_struct *win = WIN(data);
+	if((widget == NULL) || (dc == NULL) || (win == NULL))
+	    return;
+	
+	WinDragDataReceivedNexus(
+	    win, widget, dc, info, selection_data,
+	    0, 0
+	);                                         
+}
+
+
 /*
  *	Sets the DND icon based on the specified cell on the Win's
  *	GtkCList.
