@@ -61,8 +61,8 @@ void WinRecursiveCB(GtkWidget *widget, gpointer data);
 void WinExecutablesOnlyCB(GtkWidget *widget, gpointer data);
 void WinIgnoreLinksCB(GtkWidget *widget, gpointer data);
 
-static gint WinEDVRecycleProgressCB(   
-	gpointer data, gulong pos, gulong total
+static gint WinEDVRecycleProgressCB(
+	gpointer data, const gulong i, const gulong m
 );
 void WinResultsRecycleCB(GtkWidget *widget, gpointer data);
 void WinResultsMoveCB(GtkWidget *widget, gpointer data);
@@ -112,7 +112,7 @@ static const gchar *GET_NAME_FROM_PATH(const gchar *path)
 	if(STRISEMPTY(path))
 	    return(path);
 
-	s = strrchr(path, DIR_DELIMINATOR);
+	s = strrchr(path, G_DIR_SEPARATOR);
 	return((s != NULL) ? (s + 1) : path);
 }
 
@@ -835,7 +835,7 @@ void WinPasteCB(GtkWidget *widget, gpointer data)
 
 	    gtk_clist_thaw(clist);
 
-	    /* Delete object list */
+	    /* Delete the object list */
 	    g_list_foreach(obj_list, (GFunc)ObjDelete, NULL);
 	    g_list_free(obj_list);
 	}
@@ -906,9 +906,10 @@ void WinInvertSelectionCB(GtkWidget *widget, gpointer data)
 void WinStartCB(GtkWidget *widget, gpointer data)
 {
 	gint i;
+	gchar *location, **strv;
+	GList *paths_list;
 	GtkWidget *toplevel;
 	GtkCList *clist;
-	gchar *location;
 	avscan_options options;
 	obj_struct *obj;
 	core_struct *core;
@@ -921,17 +922,50 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 
 	if((widget != NULL) ? !GTK_WIDGET_SENSITIVE(widget) : FALSE)
 	    return;
-		   
-	WinSetBusy(win, TRUE);
 
 	toplevel = win->toplevel;
 	clist = GTK_CLIST(win->scan_clist);
 	core = CORE(win->core);
 
-	/* Get scan values */
+	/* Is there a scan or update already in progress? */
+	if((win->scan_toid != 0) || (win->update_toid != 0))
+	{
+#ifdef HAVE_EDV2
+            EDVPlaySoundWarning(core->edv_ctx);
+#endif
+            CDialogSetTransientFor(toplevel);
+            CDialogGetResponse(
+                "Scan Failed",
+"A scan or virus database update is already in progress.\n\
+\n\
+You may either wait for the current scan or update to\n\
+complete itself or click on the stop button to stop the\n\
+current scan or update before starting a new scan.",
+                NULL,
+                CDIALOG_ICON_WARNING,
+                CDIALOG_BTNFLAG_OK,
+                CDIALOG_BTNFLAG_OK
+            );
+            CDialogSetTransientFor(NULL);
+	    return;
+	}
+
+	WinSetBusy(win, TRUE);
+
+	/* Get the scan values */
 	location = STRDUP(
 	    gtk_entry_get_text(GTK_ENTRY(win->location_entry))
 	);
+	strv = g_strsplit(location, ",", -1);
+	paths_list = NULL;
+	if(strv != NULL)
+	{
+	    gint i;
+	    for(i = 0; strv[i] != NULL; i++)
+		paths_list = g_list_append(paths_list, STRDUP(strv[i]));
+	    g_strfreev(strv);
+	}
+
 	options = 0;
 	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(win->recursive_check))
 	    options |= AVSCAN_OPT_RECURSIVE;
@@ -957,14 +991,17 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	    ); 
 	    CDialogSetTransientFor(NULL);
 
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
+	    g_free(location);
+
 	    WinSetBusy(win, FALSE);
-	    free(location);
 	    return;
 	}
 
 
-	/* Get selected row and see if the scan item object's location
-	 * matches the specified location
+	/* Get the selected row and see if the scan item object's
+	 * location matches the specified location
 	 */
 	for(i = 0; i < clist->rows; i++)
 	{
@@ -984,8 +1021,9 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	/* Start scan */
 	WinScanProcessStart(
 	    win,
-	    NULL,		/* Database Path */
-	    location,		/* Path to object to scan */
+	    NULL,		/* Use the already database path from
+				 * the configuration */
+	    paths_list,		/* List of objects to scan */
 	    options
 	);
 
@@ -994,6 +1032,8 @@ void WinStartCB(GtkWidget *widget, gpointer data)
 	    GTK_NOTEBOOK(win->notebook), (gint)WIN_PAGE_NUM_RESULTS
 	);
 
+	g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	g_list_free(paths_list);
 	g_free(location);
 
 	WinSetBusy(win, FALSE);
@@ -1027,6 +1067,7 @@ void WinStopCB(GtkWidget *widget, gpointer data)
 void WinLocationCB(GtkWidget *widget, gpointer data)
 {
 	gboolean status;
+	const gchar *prev_location, *s;
 	GtkWidget *toplevel;
 	GtkEntry *entry;
 	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
@@ -1046,6 +1087,11 @@ void WinLocationCB(GtkWidget *widget, gpointer data)
 	toplevel = win->toplevel;
 	entry = GTK_ENTRY(win->location_entry);
 
+	prev_location = gtk_entry_get_text(entry);
+	s = strrchr(prev_location, ',');
+	if(s != NULL)
+	    prev_location = s + 1;
+
 	WinSetBusy(win, TRUE);
 
 	/* Create file types list */
@@ -1059,7 +1105,7 @@ void WinLocationCB(GtkWidget *widget, gpointer data)
 	status = FileBrowserGetResponse(
 	    "Select Location",
 	    "Select", "Cancel",
-	    gtk_entry_get_text(entry),
+	    prev_location,
 	    ftype, total_ftypes,
 	    &path_rtn, &total_path_rtns,
 	    &ftype_rtn
@@ -1069,10 +1115,28 @@ void WinLocationCB(GtkWidget *widget, gpointer data)
 	/* Got user response? */
 	if(status)
 	{         
-	    const gchar *path = (total_path_rtns > 0) ?   
-		path_rtn[0] : NULL;
-	    if(!STRISEMPTY(path))
-		gtk_entry_set_text(entry, path);
+	    gint i;
+	    gchar *s, *location = STRDUP("");
+	    const gchar *path;
+
+	    for(i = 0; i < total_path_rtns; i++)
+	    {
+		path = path_rtn[i];
+		if(STRISEMPTY(path))
+		    continue;
+
+		s = g_strconcat(
+		    location,
+		    (i > 0) ? "," : "",
+		    path,
+		    NULL
+		);
+		g_free(location);
+		location = s;
+	    }
+
+	    gtk_entry_set_text(entry, location);
+	    g_free(location);
 	}
 
 	/* Delete file types list */
@@ -1191,7 +1255,7 @@ void WinIgnoreLinksCB(GtkWidget *widget, gpointer data)
  *	Endeavour Recycle progress callback.
  */
 static gint WinEDVRecycleProgressCB(
-	gpointer data, gulong pos, gulong total
+	gpointer data, const gulong i, const gulong m
 )
 {
 	win_struct *win = WIN(data);
@@ -1200,7 +1264,7 @@ static gint WinEDVRecycleProgressCB(
 
 	WinStatusProgress(
 	    win,
-	    (total > 0) ? (gfloat)pos / (gfloat)total : 0.0f,
+	    (m > 0l) ? (gfloat)i / (gfloat)m : 0.0f,
 	    TRUE
 	);
 
@@ -1326,13 +1390,14 @@ void WinResultsRecycleCB(GtkWidget *widget, gpointer data)
 		if(path != NULL)
 		{
 		    /* Recycle this object */
-		    guint index = EDVRecycle(
+		    const guint index = EDVRecycle(
 			core->edv_ctx, path,
 			TRUE,
 			WinEDVRecycleProgressCB, win
 		    );
 		    if(index != 0)
 		    {
+			EDVPlaySoundCompleted(core->edv_ctx);
 			EDVContextSync(core->edv_ctx);
 			gtk_clist_remove(clist, row);
 		    }
@@ -1958,9 +2023,7 @@ void WinDBUpdateNetCB(GtkWidget *widget, gpointer data)
 
 	WinSetBusy(win, TRUE);
 
-	/* Get virus database update values */
-
-	/* Start virus database update */
+	/* Start the virus database update */
 	WinUpdateProcessStart(win);
 
 	WinSetBusy(win, FALSE);
